/*
 * PeakLog.java
 *
 * Created on 5 March 2006, 12:43
 */

package log;

import java.io.*;
import java.util.*;
//import optimizer.tools.*;
/**
 * Generates a log file containing time, fitness, and peak detection
 * information.
 *
 * @author  mike
 * @version 0.1
 */
public class PeakLog {
    
        

    /** current filename */
    protected String filename = "default.log";
    
    /** output file */
    protected FileWriter out = null;
    
    /** print format for doubles */
    protected String numF  = "%13.5G";
    
    /** print format for longs */
    protected String longF = "%13d";
    
    /** record the beginning of the optimization */
    protected long startTime = System.currentTimeMillis();

    /** keeps the best of all fitness values */
    protected double allBestFit = -10e10;
    
    
    /**
     * Creates a new instance of PeakLog that
     * writes onto the given file.
     */
    public PeakLog( String filename ) {
        
        this.filename = filename;
        try{
            out = new FileWriter( filename );
        } catch( IOException ex ) {
            ex.printStackTrace();
            out = null;
        }
        
    }
    
    /** Writes parameters as comments into the log */
    public void writeLogHeader( Properties origConfig ) {
        
            try{
                out.write( "# Experiment properties:\n#\n" );
                Enumeration keys = origConfig.keys();
                Vector sortedKeys = new Vector();
                while( keys.hasMoreElements() )
                    sortedKeys.add( (String)keys.nextElement() );
                Collections.sort( sortedKeys );
                String key;
                for( int k=0; k < sortedKeys.size(); k++ ) {
                    key = (String)sortedKeys.get( k );
                    //while( keys.hasMoreElements() ) {
                    //    key = (String)keys.nextElement();
                    out.write( "# " + key + " = "
                    + origConfig.getProperty( key ) + "\n" );
                }
            } catch( IOException ex ) { ex.printStackTrace(); }
    }
            
    /** write one-line header for table fields */
    public void writeHeader() {
                        
        try{
            out.write("#\n# timestep(ms)        evals   generations    allBestFit       bestFit    currentFit"
                    +"    peaksFound    numOfPeaks  peakFraction \n");
        } catch( IOException ex ) {
            ex.printStackTrace();
        }
    }    

    /** write one-line alternative header for table fields */
    public void writeHeader(String header) {
                        
        try{
            out.write( header ); 
        } catch( IOException ex ) {
            ex.printStackTrace();
        }
    }    
    
    
    /** write a log line concerning the current state of the population. */
    public void writeLog( long evals, long generations, double bestFit, double currentFit,
            int peaksFound, int numOfPeaks ) {
        
        long curTime = System.currentTimeMillis();
        //System.out.println("log-time: " + curTime );
        Formatter f = new Formatter();
               
        double peakFraction = 0;
        if( numOfPeaks > 0 ) peakFraction = (double)peaksFound/(double)numOfPeaks;
        if( bestFit > this.allBestFit ) allBestFit = bestFit;
        try{
            f.format( longF + " " + longF + " " + longF + " " + numF + " " 
                    + numF + " "  + numF + " " + longF + " " + longF + " " + numF + " ",
                    curTime-startTime, evals, generations, allBestFit, bestFit, currentFit, peaksFound,
                    numOfPeaks, peakFraction );            
            out.write( f.toString() + "\n" );
            out.flush();
        } catch( IOException ex ) {
            ex.printStackTrace();
        }
    }         
    
        
    /** closes the underlying file writer */
    public void close() {
        
        try{
            out.close();
        } catch( IOException ex ) {
            ex.printStackTrace();
        }
    }
    
    
}
