package multiNationSpecies;

import java.io.File;
import java.util.Properties;
import java.util.Formatter;
import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.util.StringTokenizer;
import java.util.Vector;

import log.Tools;
import log.PeakLog;
import functions.test_func;
import functions.benchmark;

public class Main
{
    private int functionNo = 14;//which function we are optimizing
    //1 = De Jong
    //2 = Schwefel

    //from multinationals
    //3 = 5 hills, 4 valleys - Ursem F3
    //4 = 1 center peak and four neighbours - Ursem F4
    //5 = Six Hump Camel Back
    //6 = Waves

    //from species conservation
    //7 = Deb - modify the "noOfGenes" from Individual.java to 1
    //9 = 2-peak trap
    //10 = central 2-peak trap
    //11 = 5 uneven peak trap

    //12 = Rastrigin
    //13 = Skewbi
    //14 Branin RCOS
    //15 Ackley
    //16 Shubert
    //17 Michalewicz
    //18 Ursem F1 - 2 peaks, 1 global, 1 local
    //19 Rescaled 6-hump
    
    //CEC 2005 benchmark functions (add 13 to the curent no of the function in CEC)
    //22 = Shifted Rastrigin
    //34 = Rotated Hybrid Composition Function 3
    
    private int popSize = 2;//no of chromosomes in the first population - read from parametersfile.txt
    private Individual[] c;//the chromosomes that are forming the population
    private Individual[] seeds;

    private double[] vector;
    private int vectorSize = 6;//for hill_valley in order to find the interior points - read from parametersfile.txt
    private double pc = 0;//crossover probability - read from parametersfile.txt
    private double pm = 0;//mutation probability - read from parametersfile.txt
    private double mutationStrength = 0; //- read from parametersfile.txt
    //Next variable is used for selecting the solutions	among the seeds in "printResults"
    private int noOfVariables = 0;//for how many variables is the function considered - read from parametersfile.txt
    private int generationNo = 0;//number of the current generation
    private int tournamentSize = 0;//how many individuals participate in a tournament for mating selection - read from parametersfile.txt
    private int fitnessEvaluationsTotal = 0, fitnessEvaluations = 0; // fitnessEvaluationTotal - read from parametersfile.txt
    //for highly multimodal functions, one has to stop the increase of the number of seeds
    private int maximumNoOfSeeds = 0;//population size * (percentSeeds/100)
    private int percentSeeds = 20;
    private int noOfRuns = 1;

    //for measuring how good the results are
    private int allPeaksFound = 0;//number of runs in which all the peaks are found
    private int averagePeaksFound = 0;//average number of peaks found

    private int noOfOptimaPeaks = 0;
    private Individual[] optimaPeaks;
    //if the value found is lower than the actual
    //optimum of the function with less than eps,
    //then the optimum is said to be found
    private boolean useInteriorPoints = false; //- read from parametersfile.txt
    private boolean showInteriorPoints = false;//for the hillValley to print interior points. It is modified in verifyHillValley
    private int selType = -1;//Type of selection (0 - global; 1- local) - read from parametersfile.txt
    private double eps = 0.1;//how close to the optimum should our value be in order to be consider the optimum found
    private double bestFit = -10e10, currentFit = -10e10;
    
    private double elapsedTime = 0;
    private boolean replaceParentAutomaticallyMutation = true; //if true, the offspring obtained after mutation replaces the parent even if not better - read from parametersfile.txt
    private int evalNo = 100;
    private double bestEvaluations[] = new double[evalNo];

    /** holds the algorithms parameters */
    public Properties parameters = null;

    /** Property key for the current run property used for batch ea runs */
    public static final String CURRENT_RUN_PROP_KEY = "currentRun";
    int duplicatesAvoided = 0, seedsInserted = 0, timesFreeIndividuals = 0;
    int fromGeneration = 0, toGeneration = 0;
    private int configurationSPO = 0, seedSPO = 0, stepSPO = 0;
    private boolean useLHDs = true;
    //if SPO is used what is the task (best fitness or no of optima)
    private boolean findBestFitness = true;//if false, than the output "Y" result gives the no of optima found
    
    benchmark theBenchmark;
    test_func aTestFunc;

/**
 * @param parameters 
 * @param log 
 * @return 
 */
    public Main( Properties parameters, PeakLog log)
    {
        readParameters( parameters );
        maximumNoOfSeeds = (int)((double)popSize * ((double) percentSeeds / 100.0));        
        if(useLHDs)
            runWithLHDs();
        else
        {
            //all the computation starts here
            double startTime = (double)System.currentTimeMillis()/1000;
            int currentRun = 0;
            allPeaksFound = 0;
            averagePeaksFound = 0;
            int averageGenerations = 0;

            while(currentRun < noOfRuns)
            {
                initializePopulation();
                //System.out.println("Dupa initializare:");
                //printForR(c);
                generationNo = 0;
                bestFit = getBestFit();
                currentFit = getAverageFit();
               // log.writeLog( fitnessEvaluations, generationNo, bestFit, currentFit, peaksFound(), noOfOptimaPeaks );
                int howManySeeds = 0;
                Vector optimaVector = new Vector(1, 1);
                Vector fitVector = new Vector(1, 1);
                while(fitnessEvaluations < fitnessEvaluationsTotal)
                {
                    switch(selType)
                    {
                        case 0://global selection
                            determineSeeds(false);
                            //if((generationNo < 10) || (generationNo % 20 == 0))
                              //  printSpeciesForR(false);
                            if(generationNo == 0)
                                howManySeeds = seeds.length;
                            if(seeds.length < howManySeeds)
                            {
                                System.out.println("After determineSeeds in generation " + generationNo + " seeds no is decreased from " + 
                                    howManySeeds + " to " + seeds.length + ".");
                                howManySeeds = seeds.length;
                            }
                            tournamentSelection();
                            recombination();
                            normalMutation();
                            insertSeedsWithoutDuplicates();
                            integrateIndividualsToExistingSeeds();
                            if(seeds.length < howManySeeds)
                            {
                                System.out.println("After insertSeedsWithoutDuplicates in generation " + generationNo + " seeds no is decreased from " + 
                                    howManySeeds + " to " + seeds.length + ".");
                                howManySeeds = seeds.length;
                            }

                            if(countFreeIndividuals() > 0)
                            {
                                timesFreeIndividuals++;
                                integrateFreeIndividuals();
                                //integrateFreeIndividualsAndReorderSubpopulations();
                            }
                            if(fitnessEvaluations >= fitnessEvaluationsTotal)
                                determineSeeds(true);
                            determineBestEvaluation();
                            generationNo++;
                            bestFit = getBestFit();
                            currentFit = getAverageFit();
                            optimaVector.addElement(peaksFound());
                            fitVector.addElement(fitnessEvaluations);
                            //if(generationNo == 1) && 
                            //{
                            	//System.out.println("Gen " + generationNo + " and no of seeds = " + seeds.length);
                            	//printSpeciesForR(true);
                            //}
                            //System.out.println("Gen " + generationNo + " and no of seeds = " + seeds.length);
                           // log.writeLog( fitnessEvaluations, generationNo, bestFit, currentFit, peaksFound(), noOfOptimaPeaks );
                            break;
                        case 1://local selection
                            if(generationNo > 0)
                                howManySeeds = seeds.length;
                            determineSeeds(false);
                            System.out.println("No of seeds in generation " + generationNo + ": " + seeds.length);
                            //if((generationNo < 5) || (generationNo % 5 == 0))
                              //  printSpeciesForR(false);
                            if((generationNo == 1) || (generationNo == 0))
                                    howManySeeds = seeds.length;
                            if((generationNo > fromGeneration) && (toGeneration > generationNo))
                                    System.out.println("Gen " + generationNo + " dupa detseeds " + fitnessEvaluations);

                            if(seeds.length < howManySeeds)
                            {
                                System.out.println("After determineSeeds in generation " + generationNo + " seeds no is decreased from " + 
                                                howManySeeds + " to " + seeds.length + ".");
                                howManySeeds = seeds.length;
                            }
                            localTournamentSelectionAndRecombination();
                            if((generationNo > fromGeneration) && (toGeneration > generationNo))
                                System.out.println("Gen " + generationNo + " dupa sel si recombinare " + fitnessEvaluations);
                            normalMutation();
                            if((generationNo > fromGeneration) && (toGeneration > generationNo))
                                System.out.println("Gen " + generationNo + " dupa mutatie " + fitnessEvaluations);
                            insertSeedsWithoutDuplicates();
                            if((generationNo > fromGeneration) && (toGeneration > generationNo))
                                    System.out.println("Gen " + generationNo + " dupa inserarea seeds " + fitnessEvaluations);
                            integrateIndividualsToExistingSeeds();
                            if((generationNo > fromGeneration) && (toGeneration > generationNo))
                                System.out.println("Gen " + generationNo + " dupa integrarea inds la seeds " + fitnessEvaluations);
                            if(countFreeIndividuals() > 0)
                            {
                                timesFreeIndividuals++;
                                integrateFreeIndividuals();
                                //integrateFreeIndividualsAndReorderSubpopulations();
                                if((generationNo > fromGeneration) && (toGeneration > generationNo))
                                        System.out.println("Gen " + generationNo + " dupa reord pop " + fitnessEvaluations);
                            }

                            if(fitnessEvaluations >= fitnessEvaluationsTotal)
                                determineSeeds(true);
                            determineBestEvaluation();
                            generationNo++;
                            bestFit = getBestFit();
                            currentFit = getAverageFit();
                            log.writeLog( fitnessEvaluations, generationNo, bestFit, currentFit, peaksFound(), noOfOptimaPeaks );
                            break;
                        default:
                            System.out.println("selType parameter can only be 0 or 1!");
                            fitnessEvaluations = fitnessEvaluationsTotal;
                    }
                }
                    System.out.println("\nRun " + (currentRun + 1) + " after " + generationNo +
                            	" generations and " + fitnessEvaluations + " fitness evaluations:");
                    /*
                    System.out.println("The found optima");
                    for(int k = 0; k < optimaVector.capacity(); k++)
                    	System.out.print(optimaVector.get(k) + " ");                    
                    System.out.println();
                    System.out.println("The fitness evaluations used so far");
                    for(int k = 0; k < fitVector.capacity(); k++)
                    	System.out.print(fitVector.get(k) + " ");                    
                    System.out.println();
                    */
                    averageGenerations += generationNo;
                    computeAveragePeaksFound();
                    //printForR(seeds);
                    currentRun++;
            }
            
            double stopTime = (double)System.currentTimeMillis()/1000;
            elapsedTime = stopTime - startTime;
            //printThePeaks();
            System.out.println("\n\n******************************\n");
            System.out.println("In " + noOfRuns + " runs the average number of generations was " + ((double)averageGenerations) / (double)noOfRuns);
            System.out.println("In " + allPeaksFound + " out of " + noOfRuns  +
                                           " runs, all the optima were detected! Percentage " +
                                                   100 * ((double)allPeaksFound / (double)noOfRuns) + "%");            
            System.out.println("In " + noOfRuns + " runs, we have the average number of " +
                        "optima found " + ((double)averagePeaksFound / (double)noOfRuns) +                        
                                " out of the maximum " + noOfOptimaPeaks);
          
        }
    }
    
    private String getInputFileName()
    {
    	String ret = "";
    	switch(functionNo)
    	{
    	case 1:
    		if(noOfVariables == 2)
    			ret = "Parameters intervals\\MutationStrength 0-15\\TSC\\LHD.des";
    		else
    			ret = "Parameters intervals\\MutationStrength 0-30\\TSC\\LHD.des";
    		break;
    	case 3:
    		ret = "Parameters intervals\\MutationStrength 0-5\\TSC\\LHD.des";
    		break;
    	case 4:
    		ret = "Parameters intervals\\MutationStrength 0-5\\TSC\\LHD.des";
    		break;
    	case 5:
    		ret = "Parameters intervals\\MutationStrength 0-5\\TSC\\LHD.des";
    		break;
    	case 6:
    		ret = "Parameters intervals\\MutationStrength 0-5\\TSC\\LHD.des";
    		break;
    	case 8:
    		ret = "Parameters intervals\\MutationStrength 0-5\\TSC\\LHD.des";
    		break;
    	case 12:
    		if(noOfVariables == 2)
    			ret = "Parameters intervals\\MutationStrength 0-15\\TSC\\LHD.des";
    		else
    			ret = "Parameters intervals\\MutationStrength 0-30\\TSC\\LHD.des";
    		break;
    	case 14:
    		ret = "Parameters intervals\\MutationStrength 0-15\\TSC\\LHD.des";
    		break;
    	case 15:
    		ret = "Parameters intervals\\MutationStrength 0-80\\TSC\\LHD.des";
    		break;
    	case 16:
    		ret = "Parameters intervals\\MutationStrength 0-5\\TSC\\LHD.des";
    		break;
    	case 17:
    		ret = "Parameters intervals\\MutationStrength 0-5\\TSC\\LHD.des";
    		break;
    	case 18:
    		ret = "Parameters intervals\\MutationStrength 0-5\\TSC\\LHD.des";
    		break;
    	case 19:
    		ret = "Parameters intervals\\MutationStrength 0-5\\TSC\\LHD.des";
    		break;
    	case 20:
    		ret = "Parameters intervals\\MutationStrength 0-15\\TSC\\LHD.des";
    		break;
    	case 22:
    		if(noOfVariables == 2)
    			ret = "Parameters intervals\\MutationStrength 0-15\\TSC\\LHD.des";
    		else
    			ret = "Parameters intervals\\MutationStrength 0-30\\TSC\\LHD.des";
    		break;
    	case 34:
    		if(noOfVariables == 2)
    			ret = "Parameters intervals\\MutationStrength 0-15\\TSC\\LHD.des";
    		else
    			ret = "Parameters intervals\\MutationStrength 0-30\\TSC\\LHD.des";
    		break;
    	case 50:
    		ret = "Parameters intervals\\MutationStrength 0-30\\TSC\\LHD.des";
    		break;
    	default:
    		System.out.println("The function name " + functionNo + " is not mentioned!");
    	}
    	return ret;
    }
    
    private String getFunctionName()
    {
    	String ret = "";
    	switch(functionNo)
    	{
    	case 1:
    		ret = "De Jong";
    		break;
    	case 3:
    		ret = "Ursem F3";
    		break;
    	case 4:
    		ret = "Ursem F4";
    		break;
    	case 5:
    		ret = "6-hump";
    		break;
    	case 6:
    		ret = "Waves";
    		break;
    	case 12:
    		ret = "Rastrigin";
    		break;
    	case 14:
    		ret = "Branin";
    		break;
    	case 15:
    		ret = "Ackley";
    		break;
    	case 16:
    		ret = "Shubert";
    		break;
    	case 17:
    		ret = "Michalewicz";
    		break;
    	case 18:
    		ret = "Ursem F1";
    		break;
    	case 19:
    		ret = "Shifted 6-hump";
    		break;
    	case 22:
    		ret = "Rastrigin";
    		break;
    	case 34:
    		ret = "F21";
    		break;
    	case 50:
    		ret = "Keane's Bump";
    		break; 
    	default:
    		System.out.println("The function name " + functionNo + " is not mentioned!");
    	}
    	return ret;
    }
/**
 * Next method takes the input values for the parameters from LHDs.  
 */
    private void runWithLHDs()
    {
        try//we're reading the SPO parameters
        {
        	String directoryName = getFunctionName();
        	String inputFileName = getInputFileName();
        	if(functionNo == 20)
        		inputFileName = "multiNationSpecies\\LHD.des";
        	RandomAccessFile inputFile = new RandomAccessFile(inputFileName, "r");
            
            try//write the results to result file
            {
                String fileName = "TSC\\" + directoryName + "\\f" + functionNo + "_" + noOfVariables + "vars_TSC.res";//result file
                
                FileWriter fileWriter;
                BufferedWriter buffWriter;
                
                try
                {
                    //create the result file with only the header
                    String firstLine = inputFile.readLine();
                    File testExistance = new File(fileName);
                    if(!testExistance.exists())
                    {//if the res file does not exist, create it and write the header
                        fileWriter = new FileWriter(fileName);
                        buffWriter = new BufferedWriter(fileWriter);
                        
                        if(findBestFitness)
                            buffWriter.write("averageBestFitness " + firstLine + " averagePeaksFound peakRatio basinsFound " +
                                    "allPeaksFound averagePeakAccuracy distanceAccuracy fitnessEvaluationsTotal " +
                                    "noOfVariables selType replaceParentAutomaticallyMutation " +
                                    "tournamentSize averageGenerations runtime\n");
                        else
                            buffWriter.write("averagePeaksFound " + firstLine + " averageBestFitness peakRatio basinsFound " +
                                    "allPeaksFound averagePeakAccuracy distanceAccuracy fitnessEvaluationsTotal " +
                                    "noOfVariables selType replaceParentAutomaticallyMutation " +
                                    "tournamentSize averageGenerations runtime\n");
                        buffWriter.close();
                    }
                }catch(IOException e)
                {
                    System.out.print("Exception when testing wether the res file already exists!" + e);
                }

                fileWriter = new FileWriter(fileName, true);
                buffWriter = new BufferedWriter(fileWriter);
                
                String line = inputFile.readLine();
                double averageBestFit = 0;

                while(line != null)
                {
                    StringTokenizer stringTokenizer = new StringTokenizer(line, " ");
                    popSize = (int)Math.round(Double.parseDouble( stringTokenizer.nextToken() ) );
                    pc = Double.parseDouble( stringTokenizer.nextToken() );
                    pm = Double.parseDouble( stringTokenizer.nextToken() );
                    mutationStrength = Double.parseDouble( stringTokenizer.nextToken() );
                    vectorSize = (int)Math.round(Double.parseDouble( stringTokenizer.nextToken() ) );
                    noOfRuns = Integer.parseInt(stringTokenizer.nextToken());
                    configurationSPO = Integer.parseInt(stringTokenizer.nextToken());
                    seedSPO = Integer.parseInt(stringTokenizer.nextToken());
                    stepSPO = Integer.parseInt(stringTokenizer.nextToken());
                    averageBestFit = 0;
                    maximumNoOfSeeds = (int)((double)popSize * ((double) percentSeeds / 100.0));
                    line = inputFile.readLine();

                    //all the computation starts here
                    double startTime = (double)System.currentTimeMillis()/1000;
                    int currentRun = 0;
                    allPeaksFound = 0;
                    averagePeaksFound = 0;
                    int averageGenerations = 0;
                    double averagePeakAccuracy = 0, averagePeakRatio = 0, averageBasinsFound = 0, averageDistanceAccuracy = 0;
                    while(currentRun < noOfRuns)
                    {
                        initializePopulation();
                        generationNo = 0;                        
                        bestFit = getBestFit();
                        currentFit = getAverageFit();
                        //log.writeLog( fitnessEvaluations, generationNo, bestFit, currentFit, peaksFound(), noOfOptimaPeaks );
                        int howManySeeds = 0;
                        //when the functionNo == 50, put the other stop condition
                        //while(generationNo < 1000 * noOfVariables)
                        while(fitnessEvaluations < fitnessEvaluationsTotal)
                        {
                            switch(selType)
                            {
                                case 0://global selection
                                    determineSeeds(false);
                                    if(generationNo == 0)
                                        howManySeeds = seeds.length;
                                    if(seeds.length < howManySeeds)
                                    {
                                        System.out.println("After determineSeeds in generation " + generationNo + " seeds no is decreased from " + 
                                            howManySeeds + " to " + seeds.length + ".");
                                        howManySeeds = seeds.length;
                                    }
                                    tournamentSelection();
                                    recombination();
                                    normalMutation();
                                    insertSeedsWithoutDuplicates();
                                    integrateIndividualsToExistingSeeds();
                                    if(seeds.length < howManySeeds)
                                    {
                                        System.out.println("After insertSeedsWithoutDuplicates in generation " + generationNo + " seeds no is decreased from " + 
                                            howManySeeds + " to " + seeds.length + ".");
                                        howManySeeds = seeds.length;
                                    }

                                    if(countFreeIndividuals() > 0)
                                    {
                                        timesFreeIndividuals++;
                                        integrateFreeIndividuals();
                                        //integrateFreeIndividualsAndReorderSubpopulations();
                                    }
                                    //if(fitnessEvaluations >= fitnessEvaluationsTotal)
                                      //  determineSeeds(true);
                                    determineBestEvaluation();
                                    generationNo++;                                    
                                    currentFit = getAverageFit();
                                    //log.writeLog( fitnessEvaluations, generationNo, bestFit, currentFit, peaksFound(), noOfOptimaPeaks );
                                    break;
                                case 1://local selection
                                    if(generationNo > 0)
                                            howManySeeds = seeds.length;
                                    determineSeeds(false);
                                    if((generationNo == 1) || (generationNo == 0))
                                        howManySeeds = seeds.length;
                                    if((generationNo > fromGeneration) && (toGeneration > generationNo))
                                        System.out.println("Gen " + generationNo + " dupa detseeds " + fitnessEvaluations);

                                    if(seeds.length < howManySeeds)
                                    {
                                        System.out.println("After determineSeeds in generation " + generationNo + " seeds no is decreased from " + 
                                            howManySeeds + " to " + seeds.length + ".");
                                        howManySeeds = seeds.length;
                                    }
                                    localTournamentSelectionAndRecombination();
                                    if((generationNo > fromGeneration) && (toGeneration > generationNo))
                                        System.out.println("Gen " + generationNo + " dupa sel si recombinare " + fitnessEvaluations);
                                    normalMutation();
                                    if((generationNo > fromGeneration) && (toGeneration > generationNo))
                                        System.out.println("Gen " + generationNo + " dupa mutatie " + fitnessEvaluations);
                                    insertSeedsWithoutDuplicates();
                                    if((generationNo > fromGeneration) && (toGeneration > generationNo))
                                        System.out.println("Gen " + generationNo + " dupa inserarea seeds " + fitnessEvaluations);
                                    integrateIndividualsToExistingSeeds();
                                    if((generationNo > fromGeneration) && (toGeneration > generationNo))
                                        System.out.println("Gen " + generationNo + " dupa integrarea inds la seeds " + fitnessEvaluations);
                                    if(countFreeIndividuals() > 0)
                                    {
                                        timesFreeIndividuals++;
                                        integrateFreeIndividuals();
                                        //integrateFreeIndividualsAndReorderSubpopulations();
                                        if((generationNo > fromGeneration) && (toGeneration > generationNo))
                                            System.out.println("Gen " + generationNo + " dupa reord pop " + fitnessEvaluations);
                                    }

                                    //if(fitnessEvaluations >= fitnessEvaluationsTotal)
                                            //determineSeeds(true);
                                    determineBestEvaluation();
                                    generationNo++;                                    
                                    currentFit = getAverageFit();
                                    //log.writeLog( fitnessEvaluations, generationNo, bestFit, currentFit, peaksFound(), noOfOptimaPeaks );
                                    break;
                                default:
                                    System.out.println("selType parameter can only be 0 or 1!");
                                    fitnessEvaluations = fitnessEvaluationsTotal;
                            }
                        }
                            //System.out.println("\nRun " + (currentRun + 1) + " after " + generationNo +
                                    //	" generations and " + fitnessEvaluations + " fitness evaluations:");
                            averageGenerations += generationNo;
                            averageBestFit += getBestFit();
                            
                            averagePeakAccuracy += getPeakAccuracy();
                            averagePeakRatio += peakRatio();  
                            averageBasinsFound += basinsFound(); 
                            averageDistanceAccuracy += getDistanceAccuracy();
                            //printResults();
                            computeAveragePeaksFound();
                            //for(int i = 0; i < seeds.length; i++)
                                    //printForMaple(seeds[i]);
                            //printForR(seeds);
                            currentRun++;
                            //printBestFitness();
                    }

                    double stopTime = (double)System.currentTimeMillis()/1000;
                    averagePeakAccuracy /= (double)noOfRuns;
                    averagePeakRatio /= (double)noOfRuns;
                    averageBasinsFound /= (double)noOfRuns;
                    averageDistanceAccuracy /= (double)noOfRuns;
                    elapsedTime = stopTime - startTime;
                    System.out.println("\n\n************* Config " + configurationSPO + " *****************\n");
                    System.out.println("In " + noOfRuns + " runs the average number of generations was " + ((double)averageGenerations) / (double)noOfRuns);
                    System.out.println("In " + allPeaksFound + " out of " + noOfRuns  +
                        " runs, all the optima were detected! Percentage " +
                        100 * ((double)allPeaksFound / (double)noOfRuns) + "%");
                    System.out.println("In " + noOfRuns + " runs, we have the average number of " +
                        "optima found " + ((double)averagePeaksFound / (double)noOfRuns) +
                        " out of the maximum " + noOfOptimaPeaks);


                     //here ends the computation and we print the results

                     if(findBestFitness)
                        buffWriter.write(((-1) * ((double)averageBestFit) / (double)noOfRuns) + " " + popSize + " " + pc + " " + pm + " " + mutationStrength + " " 
                                + vectorSize + " " + noOfRuns + " "
                                + configurationSPO + " " + seedSPO + " " + stepSPO + " " + ((double)averagePeaksFound / (double)noOfRuns) + " " 
                                + averagePeakRatio + " " + averageBasinsFound + " " + 100 * ((double)allPeaksFound / (double)noOfRuns) + " " 
                                + averagePeakAccuracy + " " + averageDistanceAccuracy + " " + fitnessEvaluationsTotal + " " 
                                + noOfVariables + " " + selType + " " + replaceParentAutomaticallyMutation + " " 
                                + tournamentSize + " " + ((double)averageGenerations) / (double)noOfRuns + " " + ((double)elapsedTime) / (double)noOfRuns + "\n");
                     else
                         buffWriter.write(((-1) * ((double)averagePeaksFound / (double)noOfRuns)) + " " + popSize + " " + pc + " " + pm + " " 
                                + mutationStrength + " " + vectorSize + " "+ noOfRuns + " "
                                + configurationSPO + " " + seedSPO + " " + stepSPO + " " + (averageBestFit / (double)noOfRuns) + " " 
                                + averagePeakRatio + " " + averageBasinsFound + " " + 100 * ((double)allPeaksFound / (double)noOfRuns) + " "
                                + averagePeakAccuracy + " " + averageDistanceAccuracy + " " + fitnessEvaluationsTotal + " "
                                + noOfVariables + " " + selType + " " + replaceParentAutomaticallyMutation + " " 
                                + tournamentSize + " " + ((double)averageGenerations) / (double)noOfRuns + " " + ((double)elapsedTime) / (double)noOfRuns + "\n");
            }
                buffWriter.close();
                System.out.println("File " + fileName + " is created.");
            }catch(IOException e)
            {
                System.err.println("Exception on writing to file: " + e);
            }
            inputFile.close();
        }catch(IOException e)
        {
            System.out.println("An error on reading the files with the categories!\n" + e);
        }
    }
    
    //random values are given to the genes
    private void initializePopulation()
    {
        fitnessEvaluations = 0;
        vector = new double[vectorSize];
        for(int i = 0; i < vectorSize; i++)
            vector[i] = (double)(1 + i) / (double)(vectorSize + 1);
        c = new Individual[popSize];
        for(int i = 0; i < popSize; i++)
            c[i] = new Individual(functionNo, noOfVariables);
        for (int i = 0; i < popSize; i++)
            c[i].randomInitialize();
        for(int i = 0; i < popSize; i++)
            c[i].evaluation = eval(c[i]);
        setPeaks();
        bestEvaluations = new double[evalNo];
    }

    private double eval(Individual _c)
    {
        fitnessEvaluations++;
        double ret = 0;
        switch(functionNo)
        {
            case 1:
                ret = 0;
                for(int i = 0; i < _c.noOfGenes; i++)
                    ret += -(_c.genes[i] * _c.genes[i]);
                break;
            case 2:
                ret = 0;
                for(int i = 0; i < _c.noOfGenes; i++)
                    ret += _c.genes[i] * Math.sin(Math.sqrt(Math.abs(_c.genes[i])));
                ret += -418.9829 * _c.noOfGenes;
                break;
            case 3:
                ret = Math.sin(2.2 * Math.PI *_c.genes[0] + 0.5 * Math.PI) 
                                    * ((2 - Math.abs(_c.genes[1])) / 2)
                                    * ((3 - Math.abs(_c.genes[0])) / 2) 
                                    + Math.sin(0.5 * Math.PI *_c.genes[1] *_c.genes[1] + 0.5 * Math.PI) 
                                    * ((2 - Math.abs(_c.genes[1])) / 2)
                                    * ((2 - Math.abs(_c.genes[0])) / 2);
                break;
            case 4:
                ret = 3 * Math.sin(0.5 * Math.PI *_c.genes[0] + 0.5 * Math.PI) *
                    ((2 - Math.sqrt(_c.genes[0] * _c.genes[0] + _c.genes[1] * _c.genes[1])) / 4);
                break;
            case 5:
                ret = -((4 - 2.1 * _c.genes[0]*_c.genes[0] + Math.pow(_c.genes[0], 4)/3) * _c.genes[0]*_c.genes[0] + _c.genes[0]*_c.genes[1] + 4 * (_c.genes[1]*_c.genes[1] - 1) * _c.genes[1]*_c.genes[1]);
                break;
            case 6:
                ret = Math.pow(0.3 * _c.genes[0], 3) - (_c.genes[1] * _c.genes[1] - 4.5 * _c.genes[1] * _c.genes[1]) * _c.genes[0] * _c.genes[1] - 4.7 * Math.cos(3 * _c.genes[0] - _c.genes[1] * _c.genes[1] * (2 + _c.genes[0])) * Math.sin(2.5 * Math.PI * _c.genes[0]);
                break;
            case 7:
                ret = Math.pow(Math.sin(5 * Math.PI * _c.genes[0]), 6);
                break;
            case 8:
                ret = Math.pow(_c.genes[1] - 5.1/(4 * Math.PI * Math.PI) * _c.genes[0] * _c.genes[0] + (5.0 / Math.PI) * _c.genes[0] - 6, 2)
                                         + 10 * (1 - 1.0 / (8 * Math.PI)) * Math.cos(_c.genes[0]) + 10;
                break;
            case 9:
                if(_c.genes[0] < 15)
                    ret = 160.0/15.0 * (15 - _c.genes[0]);
                else
                    ret = 40 * (_c.genes[0] - 15);
                break;
            case 10:
                if(_c.genes[0] < 10)
                    ret = 16.0 * _c.genes[0];
                else
                    if(_c.genes[0] < 15)
                        ret = 32.0 * (15 - _c.genes[0]);
                    else
                        ret = 40.0 * (_c.genes[0] - 15);
                break;
            case 11:
                if(_c.genes[0] < 2.5)
                    ret = 80.0 * (2.5 - _c.genes[0]);
                else
                    if(_c.genes[0] < 5)
                        ret = 64.0 * (_c.genes[0] - 2.5);
                else
                    if(_c.genes[0] < 7.5)
                        ret = 64.0 * (7.5 - _c.genes[0]);
                    else
                        if(_c.genes[0] < 12.5)
                            ret = 28.0 * (_c.genes[0] - 7.5);
                        else
                            if(_c.genes[0] < 17.5)
                                ret = 28.0 * (17.5 - _c.genes[0]);
                            else
                                if(_c.genes[0] < 22.5)
                                    ret = 32.0 * (_c.genes[0] - 17.5);
                                else
                                    if(_c.genes[0] < 27.5)
                                        ret = 32.0 * (27.5 - _c.genes[0]);
                                    else
                                        ret = 80.0 * (_c.genes[0] - 27.5);
                break;
            case 12:
                ret = -10 * _c.noOfGenes;
                for(int i = 0; i < _c.noOfGenes; i++)
                    ret -= _c.genes[i] * _c.genes[i] - 10 * Math.cos(2 * Math.PI * _c.genes[i]);
                break;
            case 13:
                ret = 0;
                double xx,yy,x4,y4;
                for(int i = 0; i < _c.noOfGenes; i+=2) 
                {
                    xx = _c.genes[i]*_c.genes[i];
                    x4 = xx*xx;
                    yy = _c.genes[i+1]*_c.genes[i+1];
                    y4 = yy*yy;
                    ret -= x4 + y4 - xx + yy - 10*_c.genes[i]*_c.genes[i+1] + 0.25*_c.genes[i] + 20;
                }
                // origin correction not done yet ret -= 10 * _c.noOfGenes;
                ret += _c.noOfGenes/2 * 6.834483085;
                break;
            case 14:
            	ret = -(Math.pow((_c.genes[1] - (5.1/(4 * Math.PI * Math.PI)) * _c.genes[0] * _c.genes[0] + (5.0/Math.PI) * _c.genes[0] - 6), 2) + 
            			10 * (1 - (1.0/(8 * Math.PI))) * Math.cos(_c.genes[0]) + 10);
            	break;
            case 15://Ackley            	
            	double sum1 = 0, sum2 = 0;
            	for(int i = 0; i < _c.noOfGenes; i++)
            	{
            		sum1 += _c.genes[i] * _c.genes[i];
            		sum2 += Math.cos(2 * Math.PI * _c.genes[i]);
            	}
            	ret = -(20 + Math.E - 20 * Math.exp((-0.2) * Math.sqrt((1.0/_c.noOfGenes) * sum1)) - 
            			Math.exp((1.0 / _c.noOfGenes) * sum2));
            	break;
            case 16://shubert
            	ret = -1;
            	for(int i = 0; i < _c.noOfGenes; i++)
            	{
            		double sum = 0;
            		for(int j = 1; j < 6; j++)
            			sum += j * Math.cos((j + 1) * _c.genes[i] + j);
            		ret *= sum;
            	}
            	break;
            case 17://Michalewicz
            	ret =  (Math.sin(_c.genes[0]) * Math.pow(Math.sin((_c.genes[0] * _c.genes[0]) / Math.PI), 20) + 
            	Math.sin(_c.genes[1]) * Math.pow(Math.sin((2 * _c.genes[1] * _c.genes[1]) / Math.PI), 20));
            	break;
            case 18://Ursem F1
            	ret =  Math.sin(2 * _c.genes[0] - 0.5 * Math.PI) + 1 + 2 * Math.cos(_c.genes[1]) + 0.5 * _c.genes[0];
            	break;
            case 19:
            	ret = -((4 - 2.1 * _c.genes[0]*_c.genes[0] + Math.pow(_c.genes[0], 4)/3.0) * _c.genes[0]*_c.genes[0] + 
            			10 * _c.genes[0] * _c.genes[1] + 4 * ((100 * _c.genes[1]*_c.genes[1]) - 1) * (100 * _c.genes[1]*_c.genes[1]));
                break;
            case 20:
            	double[] p = new double[101];
            	for(int i = 0; i <= 100; i++)
            		p[i] = -1 + 2 * Math.random();
            	ret = 0;
            	for(int i = 0; i <= 100; i++)
            		if((-1 > f20(_c, p[i])) || (f20(_c, p[i]) > 1))
            			ret += (1 - f20(_c, p[i])) * (1 - f20(_c, p[i]));
            	if(f20(_c, 1.2) - T2k(1.2) < 0)
            		ret += (f20(_c, 1.2) - T2k(1.2)) * (f20(_c, 1.2) - T2k(1.2));
            	if(f20(_c, -1.2) - T2k(-1.2) < 0)
            		ret += (f20(_c, -1.2) - T2k(-1.2)) * (f20(_c, -1.2) - T2k(-1.2));
            	ret *= (-1);
            	break;
            case 22:
                /*The other way of computing shifted Rastrigin
                 *************************                
                double bias = -3.3000000e+002;
                double[] o = {1.9005000e+000, -1.5644000e+000, -9.7880000e-001, -2.2536000e+000, 2.4990000e+000, -3.2853000e+000, 9.7590000e-001, -3.6661000e+000, 9.8500000e-002, -3.2465000e+000, 3.8060000e+000, -2.6834000e+000, -1.3701000e+000, 4.1821000e+000, 2.4856000e+000, -4.2237000e+000, 3.3653000e+000, 2.1532000e+000, -3.0929000e+000, 4.3105000e+000, -2.9861000e+000, 3.4936000e+000, -2.7289000e+000, -4.1266000e+000, -2.5900000e+000, 1.3124000e+000, -1.7990000e+000, -1.1890000e+000, -1.0530000e-001, -3.1074000e+000, -3.9641000e+000, -4.3387000e+000, 3.0705000e+000, 3.3205000e+000, -3.8178000e+000, -1.4980000e+000, -4.3807000e+000, 2.7110000e+000, -3.7956000e+000, -2.3627000e+000, 4.0086000e+000, -1.3728000e+000, -4.4362000e+000, -2.9183000e+000, -2.2457000e+000, 3.0650000e-001, -8.9240000e-001, -3.2364000e+000, -1.2521000e+000, 2.7198000e+000, -3.9787000e+000, 3.0678000e+000, -4.2400000e+000, -3.9580000e+000, 3.9479000e+000, 2.2030000e-001, -2.6124000e+000, 2.6498000e+000, -2.3256000e+000, -1.5383000e+000, 3.4760000e+000, 2.4462000e+000, 2.4575000e+000, 3.7409000e+000, -2.4887000e+000, 3.8555000e+000, -1.1426000e+000, 1.3389000e+000, 2.2323000e+000, 2.3137000e+000, -4.3370000e+000, 3.9260000e+000, 3.5905000e+000, -1.2858000e+000, -2.0113000e+000, 2.9087000e+000, 3.9278000e+000, 1.0812000e+000, -7.4610000e-001, 3.4740000e+000, 2.3036000e+000, -3.3781000e+000, -4.4910000e-001, 9.4020000e-001, -3.0583000e+000, -4.2165000e+000, -2.3604000e+000, 8.3640000e-001, 1.4773000e+000, -2.7292000e+000, -1.5904000e+000, 7.6960000e-001, 5.1640000e-001, 2.6576000e+000, -5.4270000e-001, 1.0358000e+000, 6.9260000e-001, -4.2775000e+000, -1.5911000e+000, -3.5775000e+000};
                
                ret = -10 * (_c.noOfGenes);
                for(int i = 0; i < _c.noOfGenes; i++)
                    ret -= (_c.genes[i] - o[i]) * (_c.genes[i] - o[i]) - 10 * Math.cos(2 * Math.PI * (_c.genes[i] - o[i]));
                ret -= bias;
                ***************************
                 */                
                
                ret = (-1) * aTestFunc.f(_c.genes);
            break;
            case 34:
                ret = (-1) * aTestFunc.f(_c.genes);
            break;
            case 50:
            	double obj = 0;
            	double suma1 = 0;
            	for (int i = 0; i < _c.noOfGenes; i++)
            		suma1 += Math.pow(Math.cos(_c.genes[i]), 4);
            	double produs1 = 1;
            	for (int i = 0; i < _c.noOfGenes; i++)
            		produs1 *= Math.pow(Math.cos(_c.genes[i]), 2);
            	double suma2 = 0;
            	for (int i = 0; i < _c.noOfGenes; i++)
            		suma2 += (i + 1) * _c.genes[i] * _c.genes[i];
            	obj = (double)Math.abs(suma1 - 2 * produs1)/(double)Math.sqrt(suma2);
            	
            	double produs2 = 1;
            	for (int i = 0; i < _c.noOfGenes; i++)
            		produs2 *= _c.genes[i];
            	
            	double suma3 = 0;
            	for (int i = 0; i < _c.noOfGenes; i++)
            		suma3 += _c.genes[i];
            	double interm = (double)(15 * _c.noOfGenes)/(double)2;
            	
            	if (produs2 - 0.75 < 0 || interm - suma3 < 0) ret = 0;
            	else ret = obj;
            break;

            default:
                System.out.println("This function is not defined!");
            }
            return ret;
    }
    
    private double f20(Individual _c, double z)
    {
    	double ret = 0;
    	for(int i = 0; i < _c.noOfGenes; i++)
    		ret += _c.genes[i] * Math.pow(z, i);
    	return ret;
    }
    
    private double T2k(double z)
    {
    	double ret = 0;
    	if(c[0].noOfGenes == 9)
    		ret = 1 - 32 * z * z + 160 * Math.pow(z, 4) - 256 * Math.pow(z, 6) 
    		+ 128 * Math.pow(z, 8);
    	else
    		if(c[0].noOfGenes == 17)
    			ret = 1-128 * z * z + 2688 * Math.pow(z, 4) - 21504 * Math.pow(z, 6)
    			+ 84480 * Math.pow(z, 8) - 180224 * Math.pow(z, 10)
    			+ 212992 * Math.pow(z, 12) - 131072 * Math.pow(z, 14)
    			+ 32768 * Math.pow(z, 16);
    		else
    			System.out.println("T2k --> For F20, the number of genes has to be 8+1 or 16+1!!");
    	return ret;
    }

    private void tournamentSelection()
    {
            Individual[] newPopulation = new Individual[popSize];
            for(int i = 0; i < popSize; i++)
                    newPopulation[i] = new Individual(functionNo, noOfVariables);
            double worstFit = getWorstFit();
            for(int i = 0; i < popSize; i++)
            {
                    //countSelected counts how many individuals follow same optima as individual i
                    int[] idSelected = new int[tournamentSize], countSelected = new int[tournamentSize];
                    double[] evaluations = new double[tournamentSize];
                    for(int k = 0; k < tournamentSize; k++)
                    {
                        idSelected[k] = (int)Math.round((popSize - 1) * Math.random());
                        countSelected[k] = 0;
                    }
                    for(int k = 0; k < tournamentSize; k++)
                        for(int j = 0; j < popSize; j++)							
                            if(c[j].seedID == c[idSelected[k]].seedID)
                                countSelected[k]++;
           
                    for(int k = 0; k < tournamentSize; k++)                        
                        evaluations[k] = (1 + (c[idSelected[k]].evaluation - worstFit)) / (double)countSelected[k];
                   
                   
                    int winnerID = idSelected[0];
                    double maxEvaluation = evaluations[0];
                    for(int k = 1; k < tournamentSize; k++)
                        if(maxEvaluation < evaluations[k])
                        {
                            maxEvaluation = evaluations[k];
                            winnerID = idSelected[k];
                        }
                    
                    for(int j = 0; j < c[winnerID].noOfGenes; j++)
                        newPopulation[i].genes[j] = c[winnerID].genes[j];
                    newPopulation[i].evaluation = c[winnerID].evaluation;
                    newPopulation[i].seedID = c[winnerID].seedID;
            }
            c = new Individual[popSize];
            for(int i = 0; i < popSize; i++)
                c[i] = new Individual(functionNo, noOfVariables);
            for(int i = 0; i < popSize; i++)
            {
                    for(int j = 0; j < c[i].noOfGenes; j++)
                        c[i].genes[j] = newPopulation[i].genes[j];
                    c[i].evaluation = newPopulation[i].evaluation;
                    c[i].seedID = newPopulation[i].seedID;
            }
            /*
            if((generationNo > fromGeneration) && (toGeneration > generationNo))
            {
                    System.out.println("Generatia " + generationNo + ", populatia dupa selectie este urmatoarea:");
                    printForR(c);
            }*/
    }

    private void localTournamentSelectionAndRecombination()
    {
            /*
            System.out.println("\nPopulatia inainte de selectie");
            for(int i = 0; i < seeds.length; i++)
            {
                    for(int j = 0; j < popSize; j++)
                            if(c[j].seedID == i)
                                    printForMaple(c[j]);
                    System.out.println("\nAceasta a fost subpopulatia " + i);			
            }
            */
            Individual[] newPopulation = new Individual[popSize];
            for(int i = 0; i < popSize; i++)
                    newPopulation[i] = new Individual(functionNo, noOfVariables);
            int popSizeTemp = 0;
            for(int i = 0; i < seeds.length; i++)
            {
                    int count = 0;//to count how many individuals are from seedID "i"
                    for(int j = 0; j < popSize; j++)
                            if(c[j].seedID == i)
                                            count++;
                    if(count > 1)
                    {
                            int[] idSelected = new int[tournamentSize], realIDSelected = new int[tournamentSize];

                            for(int j = 0; j < count; j++)
                            {
                                for(int k = 0; k < tournamentSize; k++)
                                    idSelected[k] = (int)Math.round((count - 1) * Math.random());					
                                for(int k = 0; k < tournamentSize; k++)
                                    realIDSelected[k] = findID(i, idSelected[k]);
                                double maximumEvaluation = c[realIDSelected[0]].evaluation;
                                int maximumID = realIDSelected[0];
                                for(int k = 0; k < tournamentSize; k++)
                                    if(maximumEvaluation < c[realIDSelected[k]].evaluation)
                                    {
                                        maximumEvaluation = c[realIDSelected[k]].evaluation;
                                        maximumID = realIDSelected[k];
                                    }
                                for(int k = 0; k < c[maximumID].noOfGenes; k++)
                                    newPopulation[popSizeTemp].genes[k] = c[maximumID].genes[k];
                                newPopulation[popSizeTemp].evaluation = c[maximumID].evaluation;
                                newPopulation[popSizeTemp++].seedID = i;
                            }
                            //next, recombination is applied to subpopulation with seedID i
                            Individual offspring = new Individual(functionNo, noOfVariables);
                            count = 0;//reuse the same variable for counting how many parents there are
                            for(int j = 0; j < noOfVariables; j++)
                                offspring.genes[j] = 0;
                            for(int j = 0; j < popSizeTemp; j++)
                                newPopulation[j].selectedForCrossover = false;
                            for(int j = 0; j < popSizeTemp; j++)
                            {
                                if(newPopulation[j].seedID == i)
                                    if(Math.random() < pc)
                                    {
                                        for(int k = 0; k < noOfVariables; k++)
                                            offspring.genes[k] += newPopulation[j].genes[k];
                                        newPopulation[j].selectedForCrossover = true;
                                        count++;
                                    }
                            }
                            if(count > 0)
                            {
                                for(int j = 0; j < noOfVariables; j++)
                                    offspring.genes[j] /= count;
                                offspring.evaluation = eval(offspring);
                                offspring.seedID = i;
                                //now, replace the worst individual with seedID i
                                int j = 0;
                                while((newPopulation[j].seedID != i) && (!newPopulation[j].selectedForCrossover))
                                    j++;
                                double worstFit = newPopulation[j].evaluation;
                                int worstID = j;
                                for(int k = j; k < popSizeTemp; k++)
                                    if(newPopulation[j].selectedForCrossover)
                                        if(worstFit > newPopulation[k].evaluation)
                                        {
                                            worstFit = newPopulation[k].evaluation;
                                            worstID = k;
                                        }
                                if(newPopulation[worstID].evaluation < offspring.evaluation)
                                {
                                    for(int k = 0; k < noOfVariables; k++)
                                        newPopulation[worstID].genes[k] = offspring.genes[k];
                                    newPopulation[worstID].evaluation = offspring.evaluation;
                                    newPopulation[worstID].seedID = offspring.seedID;
                                }
                            }
                    }
                    else//count = 1
                    {//just copy the individual with this seedID in the newPopulation
                        if(count != 1)
                                System.out.println("count = " + count);
                        int j1 = 0, foundID = -1;
                        boolean found = false;
                        while((j1 < popSize) && (!found))
                        {
                            if(c[j1].seedID == i)
                            {
                                found = true;
                                foundID = j1;
                            }
                            j1++;
                        }
                        for(int j = 0; j < noOfVariables; j++)
                                newPopulation[popSizeTemp].genes[j] = c[foundID].genes[j];
                        newPopulation[popSizeTemp].evaluation = c[foundID].evaluation;
                        newPopulation[popSizeTemp++].seedID = c[foundID].seedID;
                    }
            }
            for(int i = 0; i < popSize; i++)
            {
                for(int j = 0; j < c[i].noOfGenes; j++)
                    c[i].genes[j] = newPopulation[i].genes[j];
                c[i].seedID = newPopulation[i].seedID;
                c[i].evaluation = newPopulation[i].evaluation;
            }
            /*
            System.out.println("\n\nPopulatia dupa selectie si recombinare");
            for(int i = 0; i < seeds.length; i++)
            {
                    for(int j = 0; j < popSize; j++)
                            if(c[j].seedID == i)
                                    printForMaple(c[j]);
                    System.out.println("\nAceasta a fost subpopulatia " + i);
            }*/
    }

    /*
     * Next method finds the ID of the individual from the seed seedID
     * and the value "value" which is from 0 to count
     */
    private int findID(int seedID, int value)
    {
            int ret = 0, j1 = 0;

            boolean found = false;
            while(!found)
            {
                    if(c[j1].seedID == seedID)
                    {
                            if(ret == value)
                                    found = true;
                            else
                                    ret++;
                    }
                    j1++;
            }
            j1--;
            return j1;
    }

    private void recombination()
    {
            Individual[] parents = new Individual[popSize];
            int noOfParents = 0;
            for(int i = 0; i < popSize; i++)
                    parents[i] = new Individual(functionNo, noOfVariables);
            for(int i = 0; i < popSize; i++)
                    c[i].selectedForCrossover = false;

            for(int i = 0; i < popSize; i++)
                if(Math.random() < pc)
                {
                    for(int k = 0; k < noOfVariables; k++)
                        parents[noOfParents].genes[k] = c[i].genes[k];
                    parents[noOfParents].evaluation = c[i].evaluation;
                    parents[noOfParents].seedID = c[i].seedID;
                    c[i].selectedForCrossover = true;
                    parents[noOfParents++].formerIndex = i;
                }
            if(noOfParents % 2 != 0)
            {
                    if((Math.random() < 0.5) && (noOfParents < popSize))//add a parent
                    {
                            int newParentID;
                            do
                            {
                                    newParentID = (int)Math.round((popSize - 1) * Math.random());
                            }while(c[newParentID].selectedForCrossover == true);
                            for(int k = 0; k < noOfVariables; k++)
                                    parents[noOfParents].genes[k] = c[newParentID].genes[k];
                            parents[noOfParents].evaluation = c[newParentID].evaluation;
                            parents[noOfParents].seedID = c[newParentID].seedID;
                            parents[noOfParents].formerID = newParentID;
                            noOfParents++;
                            c[newParentID].selectedForCrossover = true;
                    }
                    else//remove a parent
                    {
                            int removeParent = (int)Math.round((noOfParents - 1) * Math.random());
                            c[parents[removeParent].formerIndex].selectedForCrossover = false;
                            for(int j = removeParent; j < noOfParents - 1; j++)
                            {
                                    for(int k = 0; k < parents[j].noOfGenes; k++)
                                            parents[j].genes[k] = parents[j + 1].genes[k];
                                    parents[j].evaluation = parents[j + 1].evaluation;
                                    parents[j].formerID = parents[j + 1].formerID;
                                    parents[j].seedID = parents[j + 1].seedID;
                            }
                            noOfParents--;
                    }
            }//made sure noOfParents was even

            Individual[] newPopulation = new Individual[popSize];
            for(int i = 0; i < popSize; i++)
                    newPopulation[i] = new Individual(functionNo, noOfVariables);

            //add to the new population the individuals that were not selected for crossover
            int count = 0;
            for(int i = 0; i < popSize; i++)
            {
                    if(c[i].selectedForCrossover == false)
                    {
                            for(int j = 0; j < c[i].noOfGenes; j++)
                                    newPopulation[count].genes[j] = c[i].genes[j];
                            newPopulation[count].evaluation = c[i].evaluation;
                            newPopulation[count].seedID = c[i].seedID;
                            count++;
                    }
            }

            for(int i = 0; i < noOfParents; i += 2)
            {
                //the offspring
                Individual off = new Individual(functionNo, noOfVariables);
                double modify = Math.random();
                for(int k = 0; k < off.noOfGenes; k++)
                        off.genes[k] = parents[i].genes[k] + modify * (double)(parents[i + 1].genes[k] - parents[i].genes[k]);
                off.evaluation = eval(off);
                off.seedID = -1;
                //if both parents have the same seedID, the offspring will inherit it
                if(parents[i].seedID == parents[i + 1].seedID)
                    off.seedID = parents[i].seedID;

                if(parents[i].evaluation > parents[i + 1].evaluation)
                {
                    for(int j = 0; j < parents[i].noOfGenes; j++)
                        newPopulation[count].genes[j] = parents[i].genes[j];
                    newPopulation[count].evaluation = parents[i].evaluation;
                    newPopulation[count].seedID = parents[i].seedID;
                    count++;
                    //delete comments below for fighting for survival between offspring and weaker parent
                    if(off.evaluation > parents[i + 1].evaluation)
                    {
                        for(int j = 0; j < off.noOfGenes; j++)
                            newPopulation[count].genes[j] = off.genes[j];
                        newPopulation[count].evaluation = off.evaluation;					
                        newPopulation[count].seedID = off.seedID;
                        /*
                        if(off.seedID == -1)
                        {
                            if(!hillValley(parents[i], off, -1))
                                off.seedID = parents[i].seedID;
                        }
                        else
                            off.seedID = parents[i + 1].seedID;*/

                        count++;
                    }
                    else
                    {
                            for(int j = 0; j < parents[i + 1].noOfGenes; j++)
                                newPopulation[count].genes[j] = parents[i + 1].genes[j];
                            newPopulation[count].evaluation = parents[i + 1].evaluation;
                            newPopulation[count].seedID = parents[i + 1].seedID;
                            count++;
                    }
                }
                else//if(eval(parents[i]) <= eval(parents[i + 1]))
                {
                    for(int j = 0; j < parents[i + 1].noOfGenes; j++)
                        newPopulation[count].genes[j] = parents[i + 1].genes[j];
                    newPopulation[count].evaluation = parents[i + 1].evaluation;
                    newPopulation[count].seedID = parents[i + 1].seedID;
                    count++;
                    if(off.evaluation > parents[i].evaluation)
                    {
                            for(int j = 0; j < off.noOfGenes; j++)
                                newPopulation[count].genes[j] = off.genes[j];
                            newPopulation[count].evaluation = off.evaluation;
                            newPopulation[count].seedID = off.seedID;
                            /*
                            if(off.seedID == -1)
                            {
                                if(!hillValley(parents[i], off, -1))
                                        off.seedID = parents[i].seedID;
                            }
                            else
                                off.seedID = parents[i + 1].seedID;
                            newPopulation[count].seedID = off.seedID;
                            */
                            count++;
                    }
                    else
                    {
                        for(int j = 0; j < parents[i].noOfGenes; j++)
                            newPopulation[count].genes[j] = parents[i].genes[j];
                        newPopulation[count].evaluation = parents[i].evaluation;
                        newPopulation[count].seedID = parents[i].seedID;
                        count++;
                    }
                }
        }
        c = new Individual[popSize];
        for(int i = 0; i < popSize; i++)
            c[i] = new Individual(functionNo, noOfVariables);
        //replace the population
        for(int i = 0; i < popSize; i++)
        {
            for(int j = 0; j < c[i].noOfGenes; j++)
                c[i].genes[j] = newPopulation[i].genes[j];
            c[i].evaluation = newPopulation[i].evaluation;
            c[i].seedID = newPopulation[i].seedID;
        }
    }

    private void normalMutation()
    {
            for(int i = 0; i < popSize; i++)
            {
                    Individual temp = new Individual(functionNo, noOfVariables);
                    for(int k = 0; k < c[i].noOfGenes; k++)
                            temp.genes[k] = c[i].genes[k];
                    temp.evaluation = c[i].evaluation;			
                    for(int j = 0; j < c[i].noOfGenes; j++)
                    {
                            do
                            {
                                    double mutateOrNot = Math.random();
                                    if(mutateOrNot < pm)
                                    {
//						first, generate u1 and u2 and compute v1, v2, s;
                                            double u1, u2, v1 = 0, v2 = 0, s = 2, x, y;
                                            while(s > 1)
                                            {
                                                    u1 = Math.random(); u2 = Math.random();
                                                    v1 = 2 * u1 - 1; v2 = 2 * u2 - 1;
                                                    s = v1 * v1 + v2 * v2;
                                            }
                                            //compute x and y
                                            x = v1 * Math.sqrt(((-2) * (Math.log(s)/Math.log(10)))/s);
                                            y = v2 * Math.sqrt(((-2) * (Math.log(s)/Math.log(10)))/s);
                                            temp.genes[j] = c[i].genes[j] + mutationStrength * (x * y);						
                                    }
                                    else
                                            temp.genes[j] = c[i].genes[j];
                            }
                            while((temp.genes[j] > temp.rightDomain[j]) || (temp.genes[j] < temp.leftDomain[j]));
                    }

                    boolean changed = false;//verify if there is any change to c[i]
                    for(int k = 0; k < noOfVariables; k++)
                            if(temp.genes[k] != c[i].genes[k])
                                    changed = true;
                    if(changed)
                    {
                            temp.evaluation = eval(temp);
                            if(!replaceParentAutomaticallyMutation)
                            {
                                    if(temp.evaluation > c[i].evaluation)
                                    {
                                            for(int k = 0; k < c[i].noOfGenes; k++)
                                                    c[i].genes[k] = temp.genes[k];
                                            c[i].evaluation = temp.evaluation;
                                            c[i].seedID = -1;
                                    }
                            }
                            else
                            {
                                    for(int k = 0; k < c[i].noOfGenes; k++)
                                            c[i].genes[k] = temp.genes[k];
                                    c[i].evaluation = temp.evaluation;
                                    c[i].seedID = -1;
                            }
                    }//end if(changed)
            }//end for(int i = 0; i < popSize; i++)
    }

    private void determineSeeds(boolean withReordering)
    {
        for(int i = 0; i < popSize; i++)
        {
            c[i].marked = false;
            c[i].markedAsSeeds = false;
            if((generationNo == 0) || withReordering)
                c[i].seedID = -1;
            c[i].resetID();
        }
        int noOfSeeds = 0;
        Individual[] temp = new Individual[popSize];
        for(int i = 0; i < popSize; i++)
            temp[i] = new Individual(functionNo, noOfVariables);
        boolean finishedProcessing = false;
        while(!finishedProcessing)
        {
            finishedProcessing = true;
            Individual best = new Individual(functionNo, noOfVariables);
            int bestID = 0;
            while((c[bestID].marked) && (bestID < popSize - 1))
                bestID++;

            for(int j = 0; j < c[bestID].noOfGenes; j++)
                best.genes[j] = c[bestID].genes[j];
            best.evaluation = c[bestID].evaluation;
            best.seedID = c[bestID].seedID;
            for(int i = bestID + 1; i < popSize; i++)
                if((c[i].marked == false) && (best.evaluation < c[i].evaluation))
                {
                    for(int j = 0; j < c[i].noOfGenes; j++)
                        best.genes[j] = c[i].genes[j];
                    bestID = i;
                    best.evaluation = c[i].evaluation;
                    best.seedID = c[i].seedID;
                }
            c[bestID].marked = true;

            if(noOfSeeds == 0)
            {
                for(int j = 0; j < best.noOfGenes; j++)
                    temp[noOfSeeds].genes[j] = best.genes[j];
                temp[noOfSeeds].evaluation = best.evaluation;
                temp[noOfSeeds].seedID = noOfSeeds;
                temp[noOfSeeds].oldSeedID = best.seedID;
                c[bestID].seedID = noOfSeeds;
                noOfSeeds++;
            }
            else//there are several seeds
            {
                boolean completelyNew = true;//true if we see that best follows an optimum different from ALL the other "seeds"

                for(int k = 0; k < noOfSeeds; k++)
                {
                    if((generationNo > 0) && (!withReordering))
                    {
                        if(best.seedID == temp[k].oldSeedID)
                        {
                            completelyNew = false;
                            c[bestID].seedID = temp[k].seedID;
                        }
                    }
                    else//it's the first generation or reordering of subpopulations is performed
                        if(!hillValley(best, temp[k], bestID))
                        {
                            completelyNew = false;
                            c[bestID].addID(temp[k].seedID);
                        }
                }
                if(completelyNew)
                {
                	if(noOfSeeds < maximumNoOfSeeds)                	
	                {
	                    for(int j = 0; j < best.noOfGenes; j++)
	                        temp[noOfSeeds].genes[j] = best.genes[j];
	                    temp[noOfSeeds].evaluation = best.evaluation;
	                    temp[noOfSeeds].seedID = noOfSeeds;
	                    temp[noOfSeeds].oldSeedID = best.seedID;
	                    c[bestID].seedID = noOfSeeds;
	                    c[bestID].markedAsSeeds = true;
	                    noOfSeeds++;
	                }
                	else
                	{//integrate best to the closest seed
                		double minimumDistance = distance(c[bestID], temp[0]);
                		int minID = 0;
                		for(int j = 1; j < noOfSeeds; j++)
                		{
                			double newDistance = distance(c[bestID], temp[j]);                			
                			if(newDistance < minimumDistance)
                			{
                				minimumDistance = newDistance;
                				minID = j;
                			}
                		}
                		c[bestID].seedID = temp[minID].seedID;
                	}
                }
            }
            for(int i = 0; i < popSize; i++)
                if(!c[i].marked)
                    finishedProcessing = false;
        }
        seeds = new Individual[noOfSeeds];
        for(int i = 0; i < noOfSeeds; i++)
            seeds[i] = new Individual(functionNo, noOfVariables);
        for(int i = 0; i < noOfSeeds; i++)
        {
            for(int j = 0; j < temp[i].noOfGenes; j++)
                seeds[i].genes[j] = temp[i].genes[j];
            seeds[i].evaluation = temp[i].evaluation;
            seeds[i].seedID = temp[i].seedID;
        }
        if((generationNo == 0) || withReordering)
        {
            for(int i = 0; i < popSize; i++)
            {
                if(!c[i].markedAsSeeds)
                {
                    for(int j = 0; j < seeds.length; j++)
                        if(!containsSeed(c[i], seeds[j].seedID))
                            if(!hillValley(c[i], seeds[j], i))
                                c[i].addID(seeds[j].seedID);
                }
            }
            assignIDs();
        }
    }

    /*
     * Returns true if individual ind does not already contain seedID
     * in the vector of IDs
     */
    private boolean containsSeed(Individual ind, int seedID)
    {
        boolean ret = false;
        for(int i = 0; i < ind.IDs.length; i++)
            if(ind.IDs[i] == seedID)
                ret = true;
        return ret;
    }

    private void insertSeedsWithoutDuplicates()
    {
        for(int i = 0; i < popSize; i++)
            c[i].marked = false;
        //System.out.println("insertSeedsWithoutDupliactes, generation " + generationNo + " and there are " + seeds.length + " seeds.");
        //insert the seeds
        for(int i = 0; i < seeds.length; i++)
        {
            //verify whether seed i already exists in the population
            int j1 = 0;
            boolean alreadyExist = false;
            while((j1 < popSize) && (alreadyExist == false))
            {
                if((c[j1].seedID == seeds[i].seedID) && (alreadyExist == false))
                {
                    boolean identical = true;
                    int k = 0;
                    if(seeds[i].evaluation != c[j1].evaluation)
                        identical = false;
                    else
                        while ((k < seeds[i].noOfGenes) && (identical))
                        {
                            if(seeds[i].genes[k] != c[j1].genes[k])
                                identical = false;
                            k++;
                        }
                    if(identical)
                        alreadyExist = true;
                }
                j1++;
            }
            if(alreadyExist)
            {
                //System.out.println("It seems like seed " + i + " is already in the population as chr cu seedID-ul " + c[j1 - 1].seedID);
                //printForR(c[j1 - 1]);
                c[j1 - 1].marked = true;
                duplicatesAvoided++;
                /*
                for(int j = 0; j < popSize; j++)
                    if(c[j].seedID == -1)
                        if(!hillValley(seeds[i], c[j], j))
                            //c[j].addID(seeds[i].seedID);
                            c[j].seedID = seeds[i].seedID;*/
            }
            else//conserve the seed as it does not exist in the current population
            {
                seedsInserted++;
                //seeds[i].evaluation = eval(seeds[i]);
                int noOfNeighbours = 0;
                Individual[] temp = new Individual[popSize];
                for(int j = 0; j < popSize; j++)
                    temp[j] = new Individual(functionNo, noOfVariables);

                for(int j = 0; j < popSize; j++)
                {
                    if(c[j].seedID == seeds[i].seedID)
                    {
                        for(int k = 0; k < c[j].noOfGenes; k++)
                            temp[noOfNeighbours].genes[k] = c[j].genes[k];
                        temp[noOfNeighbours].evaluation = c[j].evaluation;
                        temp[noOfNeighbours++].formerID = j;
                    }/*
                    else
                        if(c[j].seedID == -1)
                            if(!hillValley(seeds[i], c[j], j))
                            {
                                for(int k = 0; k < c[j].noOfGenes; k++)
                                    temp[noOfNeighbours].genes[k] = c[j].genes[k];
                                temp[noOfNeighbours].evaluation = c[j].evaluation;
                                temp[noOfNeighbours++].formerID = j;
                                c[j].seedID = seeds[i].seedID;
                                //c[j].addID(seeds[i].seedID);
                            }*/
                }
                Individual[] temp1 = new Individual[noOfNeighbours];
                for(int j = 0; j < noOfNeighbours; j++)
                    temp1[j] = new Individual(functionNo, noOfVariables);
                for(int s = 0; s < noOfNeighbours; s++)
                {
                    for(int k = 0; k < temp[s].noOfGenes; k++)
                        temp1[s].genes[k] = temp[s].genes[k];
                    temp1[s].evaluation = temp[s].evaluation;
                    temp1[s].formerID = temp[s].formerID;
                }
                if(noOfNeighbours > 0)
                {
                    //System.out.println("Seed " + i + " are " + noOfNeighbours + " vecini.");
                    //printForR(temp1);
                    Individual worst = new Individual(functionNo, noOfVariables);
                    for(int k = 0; k < temp1[0].noOfGenes; k++)
                        worst.genes[k] = temp1[0].genes[k];
                    worst.formerID = temp1[0].formerID;
                    worst.evaluation = temp1[0].evaluation;

                    for(int j = 1; j < noOfNeighbours; j++)
                        if(worst.evaluation > temp1[j].evaluation)
                        {
                            for(int k = 0; k < temp1[j].noOfGenes; k++)
                                worst.genes[k] = temp1[j].genes[k];
                            worst.evaluation = temp1[j].evaluation;
                            worst.formerID = temp1[j].formerID;
                        }
                    if(worst.evaluation < seeds[i].evaluation)
                    {
                        for(int j = 0; j < c[worst.formerID].noOfGenes; j++)
                            c[worst.formerID].genes[j] = seeds[i].genes[j];
                        c[worst.formerID].evaluation = seeds[i].evaluation;
                    }
                    c[worst.formerID].seedID = seeds[i].seedID;
                    c[worst.formerID].marked = true;
                    //System.out.println("Cel inlocuit este cromozomul " + worst.formerID);
                    //printForR(c[worst.formerID]);
                }
                else//if there are no neighbours for seeds[i]
                {
                    Individual worst = new Individual(functionNo, noOfVariables);
                    int unmarkedID = 0;
                    while((c[unmarkedID].marked) && (unmarkedID < popSize))
                        unmarkedID++;
                    for(int k = 0; k < worst.noOfGenes; k++)
                        worst.genes[k] = c[unmarkedID].genes[k];
                    worst.evaluation = c[unmarkedID].evaluation;
                    worst.formerID = unmarkedID;
                    for(int j = unmarkedID + 1; j < popSize; j++)
                        if((!c[j].marked) && (worst.evaluation > c[j].evaluation))
                        {
                            for(int k = 0; k < c[j].noOfGenes; k++)
                                worst.genes[k] = c[j].genes[k];
                            worst.evaluation = c[j].evaluation;
                            worst.formerID = j;
                        }
                    for(int j = 0; j < c[worst.formerID].noOfGenes; j++)
                        c[worst.formerID].genes[j] = seeds[i].genes[j];
                    c[worst.formerID].evaluation = seeds[i].evaluation;
                    c[worst.formerID].marked = true;
                    c[worst.formerID].seedID = seeds[i].seedID;
                }//end else - there are no neighbours for seeds[i]
            }//end if(!alreadyExist)
        }//end for(int i = 0; i < seeds.length; i++)
            //assignIDs();
    }

    //integrate the free individuals each within their corresponding subpopulation with seed
    private void integrateFreeIndividuals()
    {
        if (seeds.length < maximumNoOfSeeds)
        {//if you can add more seeds
            int noOfFreeIndividuals = 0;
            for(int i = 0; i < popSize; i++)
                if(c[i].seedID == -1)
                {
                    c[i].marked = false;
                    noOfFreeIndividuals++;
                }
                else
                    c[i].marked = true;

            int noOfSeeds = 0;
            Individual[] temp = new Individual[noOfFreeIndividuals];
            for(int i = 0; i < noOfFreeIndividuals; i++)
                temp[i] = new Individual(functionNo, noOfVariables);
            boolean finishedProcessing = false;
            while(!finishedProcessing)
            {
                finishedProcessing = true;
                Individual best = new Individual(functionNo, noOfVariables);
                int bestID = 0;

                while((c[bestID].marked) && (c[bestID].seedID != -1) && (bestID < popSize - 1))
                    bestID++;

                for(int j = 0; j < c[bestID].noOfGenes; j++)
                    best.genes[j] = c[bestID].genes[j];
                best.evaluation = c[bestID].evaluation;
                best.seedID = c[bestID].seedID;

                for(int i = bestID + 1; i < popSize; i++)
                    if((c[i].marked == false) && (c[i].seedID == -1) && (best.evaluation < c[i].evaluation))
                    {
                        for(int j = 0; j < c[i].noOfGenes; j++)
                            best.genes[j] = c[i].genes[j];
                        bestID = i;
                        best.evaluation = c[i].evaluation;
                        best.seedID = c[i].seedID;
                    }
                c[bestID].marked = true;

                boolean completelyNew = true;//true if we see that best follows an optimum different from ALL the other "seeds"
                /*
                for(int k = 0; k < seeds.length; k++)
                {	
                    if(!hillValley(best, seeds[k], bestID))
                    {
                        completelyNew = false;
                        c[bestID].seedID = seeds[k].seedID;
                    }
                }*/
                if(completelyNew && (noOfSeeds > 0))//verify if current ind does not belong to a newly added seed
                {
                    for(int k = 0; k < noOfSeeds; k++)
                    {
                        if(!hillValley(best, temp[k], bestID))
                        {
                            completelyNew = false;
                            c[bestID].seedID = temp[k].seedID;
                        }
                    }
                }
                if(completelyNew)//then it is a new seed
                {
                    for(int j = 0; j < best.noOfGenes; j++)
                        temp[noOfSeeds].genes[j] = best.genes[j];
                    temp[noOfSeeds].evaluation = best.evaluation;
                    temp[noOfSeeds].seedID = seeds.length + noOfSeeds;// the ID has to be higher than existing ones
                    c[bestID].seedID = seeds.length + noOfSeeds;
                    noOfSeeds++;
                }

                for(int i = 0; i < popSize; i++)
                    if((c[i].seedID == -1) && (!c[i].marked))
                        finishedProcessing = false;
            }

            int newNoOfSeeds = noOfSeeds + seeds.length;
            Individual[] newSeeds = new Individual[newNoOfSeeds];
            for(int i = 0; i < newNoOfSeeds; i++)
                newSeeds[i] = new Individual(functionNo, noOfVariables);
            for(int j = 0; j < seeds.length; j++)
            {
                for(int k = 0; k < seeds[j].noOfGenes; k++)
                    newSeeds[j].genes[k] = seeds[j].genes[k];
                newSeeds[j].evaluation = seeds[j].evaluation;
                newSeeds[j].seedID = seeds[j].seedID;
            }

            int ii = 0;
            for(int j = seeds.length; j < newNoOfSeeds; j++)
            {
                for(int k = 0; k < temp[ii].noOfGenes; k++)
                    newSeeds[j].genes[k] = temp[ii].genes[k];
                newSeeds[j].evaluation = temp[ii].evaluation;
                newSeeds[j].seedID = temp[ii++].seedID;
            }
            /*
            int idNewSeeds = 0, idSeeds = 0, idTempSeeds = 0;
            while(idNewSeeds < newNoOfSeeds)
            {
                while ((idSeeds < seeds.length) && (idTempSeeds < noOfSeeds))
                {
                    if((seeds[idSeeds].evaluation > temp[idTempSeeds].evaluation))
                    {		
                        for(int j = 0; j < seeds[idSeeds].noOfGenes; j++)
                            newSeeds[idNewSeeds].genes[j] = seeds[idSeeds].genes[j];
                        newSeeds[idNewSeeds].evaluation = seeds[idSeeds].evaluation;
                        newSeeds[idNewSeeds++].seedID = seeds[idSeeds++].seedID;
                    }
                    else
                    {
                        for(int j = 0; j < temp[idTempSeeds].noOfGenes; j++)
                            newSeeds[idNewSeeds].genes[j] = temp[idTempSeeds].genes[j];					
                        newSeeds[idNewSeeds].evaluation = temp[idTempSeeds].evaluation;
                        newSeeds[idNewSeeds++].seedID = temp[idTempSeeds++].seedID;
                    }
                }
                while(idTempSeeds < noOfSeeds)
                {
                    for(int j = 0; j < temp[idTempSeeds].noOfGenes; j++)
                        newSeeds[idNewSeeds].genes[j] = temp[idTempSeeds].genes[j];
                    newSeeds[idNewSeeds].evaluation = temp[idTempSeeds].evaluation;
                    newSeeds[idNewSeeds++].seedID = temp[idTempSeeds++].seedID;
                }

                while(idSeeds < seeds.length)
                {
                    for(int j = 0; j < seeds[idSeeds].noOfGenes; j++)
                        newSeeds[idNewSeeds].genes[j] = seeds[idSeeds].genes[j];
                    newSeeds[idNewSeeds].evaluation = seeds[idSeeds].evaluation;
                    newSeeds[idNewSeeds++].seedID = seeds[idSeeds++].seedID;
                }
            }
             */
            
            seeds = new Individual[newNoOfSeeds];
            for(int i = 0; i < newNoOfSeeds; i++)
                seeds[i] = new Individual(functionNo, noOfVariables);
            for(int i = 0; i < newNoOfSeeds; i++)
            {
                for(int j = 0; j < newSeeds[i].noOfGenes; j++)
                    seeds[i].genes[j] = newSeeds[i].genes[j];
                seeds[i].evaluation = newSeeds[i].evaluation;
                seeds[i].seedID = newSeeds[i].seedID;
            }
        }
        else
        {//if you cannot add more seeds, just attach the -1 individuals to their closest seeds
            for(int i = 0; i < popSize; i++)
                if(c[i].seedID == -1)                    
                {
                    int minID = 0;
                    double minimumDistance = distance(c[i], seeds[0]);
                    for(int j = 1; j < seeds.length; j++)
                        if(minimumDistance > distance(c[i], seeds[j]))
                        {
                            minimumDistance = distance(c[i], seeds[j]);
                            minID = j;
                        }
                    c[i].seedID = seeds[minID].seedID;
                }
        }
    }
      
    private void integrateIndividualsToExistingSeeds()
    {
        for(int i = 0; i < popSize; i++)
            if(c[i].seedID == -1)//if ind i is offspring
            {
                for(int iq = 0; iq < seeds.length; iq++)
                    seeds[iq].found = false;
                int startSeed = 0;
                while((startSeed < seeds.length) && (c[i].seedID == -1))
                {
                    startSeed = 0;
                    while((startSeed < seeds.length) && (seeds[startSeed].found))
                        startSeed++;
                    if(startSeed < seeds.length)
                    {
                        double minimumDistance = distance(c[i], seeds[startSeed]);
                        int id = startSeed;//the id of the seed that is closer to ind i
                        for(int j = id + 1; j < seeds.length; j++)
                            if((!seeds[j].found) && (minimumDistance > distance(c[i], seeds[j])))
                            {
                                minimumDistance = distance(c[i], seeds[j]);
                                id = j;
                            }
                        seeds[id].found = true;
                        if(!hillValley(c[i], seeds[id], i))
                        {
                            c[i].seedID = seeds[id].seedID;
                        }
                    }
                }
            }
    }

    /*
     * Next method "reads" the parameters from the given properties object
     */
    private void readParameters( Properties props )
    {
            //try//we're reading the parameters
            //{
            //	FileInputStream fread = new FileInputStream( parametersFile );
            //      Properties props = new Properties();
            //      props.load( fread );

        popSize = (int)( Math.round(Double.parseDouble( props.getProperty( "popSize" ) ) ) );
        functionNo = (int)Math.round(Double.parseDouble( props.getProperty( "functionNo" ) ) );
        vectorSize = (int)Math.round(Double.parseDouble( props.getProperty( "vectorSize" ) ) );
        pc = Double.parseDouble( props.getProperty( "pc" ) );
        pm = Double.parseDouble( props.getProperty( "pm" ) );
        mutationStrength = Double.parseDouble( props.getProperty( "mutationStrength" ) );
        fitnessEvaluationsTotal = (int)Math.round(Double.parseDouble( props.getProperty( "fitnessEvaluationsTotal" ) ) );
        eps = Double.parseDouble( props.getProperty( "eps" ) );
        noOfVariables = (int)Math.round(Double.parseDouble( props.getProperty( "noOfVariables" ) ) );
        selType = (int)Math.round(Double.parseDouble( props.getProperty( "selType" ) ) );
        replaceParentAutomaticallyMutation = Boolean.parseBoolean( props.getProperty( "replaceParentAutomaticallyMutation" ) );
        useInteriorPoints = Boolean.parseBoolean( props.getProperty( "useInteriorPoints" ) );
        tournamentSize = (int)Math.round(Double.parseDouble( props.getProperty( "tournamentSize" ) ) );
        
        if((functionNo == 34) || (functionNo == 22))//if there is a benchmark function from CEC 2005
        {//14 here (for functionNo) corresponds to 1 in CEC 2005 list
            theBenchmark = new benchmark();
            aTestFunc = theBenchmark.testFunctionFactory(functionNo - 13, noOfVariables);
        }
        
        if(useLHDs)
        {
            if((functionNo == 1) || (functionNo == 2) || (functionNo == 12) || (functionNo == 13) || (functionNo == 22) || (functionNo == 34) || ((functionNo == 15)))
                findBestFitness = true;
            else
                findBestFitness = false;
        }
        

        //fread.close();
        //return props;
        //	}catch(IOException e)
        //	{
        //		System.out.println("An error on reading the files with the categories!\n" + e);
        //	}
    }

    private double getBestFit()
    {
            double ret = c[0].evaluation;
            for(int i = 1; i < popSize; i++)
                    if(ret < c[i].evaluation)
                            ret = c[i].evaluation;
            return ret;
    }

    private double getWorstFit()
    {
            double ret = c[0].evaluation;
            for(int i = 1; i < popSize; i++)
                    if(ret > c[i].evaluation)
                            ret = c[i].evaluation;
            return ret;
    }

    private double getAverageFit()
    {
            double ret = 0;
            for(int i = 0; i < popSize; i++)
                    ret += c[i].evaluation;
            ret /= (double)popSize;
            return ret;
    }

    /*
     * Returns true when ind1 and ind2 follow different optima.
     */
    private boolean hillValley(Individual ind1, Individual ind2, int replaceID)
    {
        boolean ret = false;

        boolean identicalPoints = true;
        int i1 = 0;
        while((i1 < ind1.noOfGenes) && (identicalPoints))
        {
            if(ind1.genes[i1] != ind2.genes[i1])
                identicalPoints = false;
            i1++;
        }
        if(identicalPoints)
            ret = false;
        else
        {
            double minEval = ind1.evaluation;//ind1.evaluation;
            if(minEval > ind2.evaluation)
                minEval = ind2.evaluation;
            Individual[] interiorIndividual = new Individual[vectorSize];
            for(int j = 0; j < vectorSize; j++)
                interiorIndividual[j] = new Individual(functionNo, noOfVariables);
            int i = 0;
            while((i < vectorSize) && (ret == false))
            {	
                for(int j = 0; j < ind1.noOfGenes; j++)
                    interiorIndividual[i].genes[j] = ind1.genes[j] + (ind2.genes[j] - ind1.genes[j]) * vector[i];
                interiorIndividual[i].evaluation = eval(interiorIndividual[i]);
                if(showInteriorPoints)
                {
                    System.out.println("\nInterior point " + i);
                    printForR(interiorIndividual[i], i);
                }
                if(minEval > interiorIndividual[i].evaluation)
                    ret = true;
                i++;
            }
            showInteriorPoints = false;
            if(useInteriorPoints)
                if((!ret) && (replaceID != -1))//only in case ind1 and ind2 follow the same optimum!
                {
                    double maxEvaluation = interiorIndividual[0].evaluation;
                    int maxID = 0;
                    for(int j = 0; j < vectorSize; j++)				
                        if(maxEvaluation < interiorIndividual[j].evaluation)
                        {
                            maxEvaluation = interiorIndividual[j].evaluation;
                            maxID = j;
                        }
                    if((maxEvaluation > ind1.evaluation) && (maxEvaluation > ind2.evaluation))//only in case it exceeds both
                    {
                        for(int k = 0; k < noOfVariables; k++)
                            c[replaceID].genes[k] = interiorIndividual[maxID].genes[k];
                        c[replaceID].evaluation = interiorIndividual[maxID].evaluation;
                    }
                }
        }//end the case when the two points are not identical
        return ret;
    }

    /*
     * All individuals that belong to several subpopulations
     * will be assigned to the one that is closest.
     */
    private void assignIDs()
    {
        for(int i = 0; i < popSize; i++)
        {
            if(!c[i].markedAsSeeds)
            {
                if(c[i].IDs.length == 1)
                    c[i].seedID = c[i].IDs[0];
                else
                    if(c[i].IDs.length > 1)//then find the closest seed and assign the c[i] to it
                    {
                        double t = distance(seeds[c[i].IDs[0]], c[i]);
                        int searchedID = 0;
                        for(int j = 1; j < c[i].IDs.length; j++)
                            if(t > distance(seeds[c[i].IDs[j]], c[i]))
                            {
                                t = distance(seeds[c[i].IDs[j]], c[i]);
                                searchedID = j;
                            }
                        c[i].seedID = c[i].IDs[searchedID];
                    }
            }
        }
    }

    /*
     * Next method verifies the hill-valley method for two certain points
     * Also prints the interior points between them.
     * The 2 points have to be given inside the method, with setGenes.
     * It works only for 2 variables.
     */
    private void verifyHillValley()
    {
            showInteriorPoints = true;
            Individual ind1 = new Individual(functionNo, noOfVariables);
            Individual ind2 = new Individual(functionNo, noOfVariables);

            ind1.setGenes(0.08982625627926766, -0.7125157455970663);
            ind2.setGenes(0.08982625631402183, -0.7125157455971276);
            System.out.println("We verify hillValley between");
            ind1.evaluation = eval(ind1);
            printForR(ind1, 0);		
            System.out.println(" and ");
            ind2.evaluation = eval(ind2);
            printForR(ind2, 1);		
            System.out.println("\nWhen hillValley is applied we obtain " + hillValley(ind1, ind2, 0));
    }

    private void computeAveragePeaksFound()
    {
        int noOfPeaksFound = peaksFound();
        if(noOfPeaksFound == noOfOptimaPeaks)
            allPeaksFound++;
        averagePeaksFound += noOfPeaksFound;
    }
    
	// just a copy of parts of the determineAllPeaksFound() method to make
	// the pure number available without any output
	private int peaksFound()
	{
	    for(int i = 0; i < noOfOptimaPeaks; i++)
	            optimaPeaks[i].found = false;
	
	    if( seeds != null )
	    {
	        for(int i = 0; i < seeds.length; i++)
	        {
	            //System.out.println("Suntem in cazul seed " + i);
	            //printForR(seeds[i]);
	            int foundID = 0;//ID of the optima peak that correspond to the seeds[i]
	            while((optimaPeaks[foundID].found) && (foundID < noOfOptimaPeaks - 1))
	                foundID++;
	
	            double minimumDistance = distance(optimaPeaks[foundID], seeds[i]);
	            for(int j = foundID + 1; j < noOfOptimaPeaks; j++)
	            if((!optimaPeaks[j].found) && (minimumDistance > distance(optimaPeaks[j], seeds[i])))
	            {
	                minimumDistance = distance(optimaPeaks[j], seeds[i]);
	                foundID = j;
	            }
	            //System.out.println("Cel mai aproape s-a gasit de optimul " + foundID);
	            //printForR(optimaPeaks[foundID]);
	            //System.out.println("Pana acum acest optima a fost gasit: " + optimaPeaks[foundID].found);
	            if((!optimaPeaks[foundID].found) && (Math.abs(optimaPeaks[foundID].evaluation - seeds[i].evaluation) < eps))
	                optimaPeaks[foundID].found = true;
	        }
	    }
	    int noOfPeaksFound = 0;
	    for(int i = 0; i < noOfOptimaPeaks; i++)
	        if(optimaPeaks[i].found)
	            noOfPeaksFound++;
	        return noOfPeaksFound;
	}
	
	//	= no of peaks found / no of peaks desired
	private double peakRatio()
	{
		return peaksFound() / noOfOptimaPeaks; 
	}
	
	//a peak is considered to be found if there is an individual in its attraction basin
	private int basinsFound()
	{
		//store the actual number of interior points 
		int actualVectorSize = vectorSize;
		vectorSize = 10;
		vector = new double[vectorSize];
        for(int i = 0; i < vectorSize; i++)
            vector[i] = (double)(1 + i) / (double)(vectorSize + 1);
        
	    for(int i = 0; i < noOfOptimaPeaks; i++)
	            optimaPeaks[i].found = false;
	
	    if( seeds != null )
	    {
	        for(int i = 0; i < seeds.length; i++)
	        {
	            //System.out.println("Suntem in cazul seed " + i);
	            //printForR(seeds[i]);
	            int foundID = 0;//ID of the optima peak that correspond to the seeds[i]
	            while((optimaPeaks[foundID].found) && (foundID < noOfOptimaPeaks - 1))
	                foundID++;
	
	            double minimumDistance = distance(optimaPeaks[foundID], seeds[i]);
	            for(int j = foundID + 1; j < noOfOptimaPeaks; j++)
	            if((!optimaPeaks[j].found) && (minimumDistance > distance(optimaPeaks[j], seeds[i])))
	            {
	                minimumDistance = distance(optimaPeaks[j], seeds[i]);
	                foundID = j;
	            }
	            //System.out.println("Cel mai aproape s-a gasit de optimul " + foundID);
	            //printForR(optimaPeaks[foundID]);
	            //System.out.println("Pana acum acest optima a fost gasit: " + optimaPeaks[foundID].found);
	            if((!optimaPeaks[foundID].found) && (!hillValley(optimaPeaks[foundID], seeds[i], foundID)))
	                optimaPeaks[foundID].found = true;
	        }
	    }
	    //restore the vectorSize
	    vectorSize = actualVectorSize;			
		vector = new double[vectorSize];
        for(int i = 0; i < vectorSize; i++)
            vector[i] = (double)(1 + i) / (double)(vectorSize + 1);
	    
	    int noOfPeaksFound = 0;
	    for(int i = 0; i < noOfOptimaPeaks; i++)
	        if(optimaPeaks[i].found)
	            noOfPeaksFound++;
	    return noOfPeaksFound;
        
	}
	
	/*
     * getPeakAccuracy() computes the accuracy find the nearest individual i and add the
     * absolute difference in fitness values between the peak center
     * and the individual i
     */
	private double getPeakAccuracy()
	{
		double ret = 0;
		
    	for(int j = 0; j < noOfOptimaPeaks; j++)
    	{
	        double minimumDistance = distance(optimaPeaks[j], c[0]);
	        int IDc = 0;
	        for(int i = 1; i < c.length; i++)
	        {
	        	double newDistance = distance(optimaPeaks[j], c[i]);
	        	if(minimumDistance > newDistance)
	        	{
	                minimumDistance = newDistance;
	                IDc = i;
	        	}
	        }
	        ret += Math.abs(optimaPeaks[j].evaluation - c[IDc].evaluation);;
    	}    	
		return ret;
	}
	
	private double getDistanceAccuracy()
	{
		double ret = 0;
		
    	for(int j = 0; j < noOfOptimaPeaks; j++)
    	{
	        double minimumDistance = distance(optimaPeaks[j], c[0]);	        
	        for(int i = 1; i < c.length; i++)
	        {
	        	double newDistance = distance(optimaPeaks[j], c[i]);
	        	if(minimumDistance > newDistance)
	                minimumDistance = newDistance;
	        }
	        ret += minimumDistance;
    	}
		return ret;
	}
	
	private double distance(Individual c1, Individual c2) 
	{			
	    double sim = 0;
	    for (int k = 0; k < c1.noOfGenes; k++)
	        sim += (c1.genes[k] - c2.genes[k]) * (c1.genes[k] - c2.genes[k]);
	    sim = Math.sqrt(sim);
	    return sim;
	}

    /*
     * Next method retains the peaks for the current function
     */
    private void setPeaks()
    {
        switch(functionNo)
        {
            case 1:
                noOfOptimaPeaks = 1;
                optimaPeaks = new Individual[noOfOptimaPeaks];
                for(int i = 0; i < noOfOptimaPeaks; i++)
                        optimaPeaks[i] = new Individual(functionNo, noOfVariables);
                for(int i = 0; i < noOfVariables; i++)
                        optimaPeaks[0].genes[i] = 0.0;
                break;
            case 2:
                noOfOptimaPeaks = 1;
                optimaPeaks = new Individual[noOfOptimaPeaks];
                for(int i = 0; i < noOfOptimaPeaks; i++)
                        optimaPeaks[i] = new Individual(functionNo, noOfVariables);
                for(int i = 0; i < noOfVariables; i++)
                        optimaPeaks[0].genes[i] = 420.9687;
                break;
            case 3:
                noOfOptimaPeaks = 5;
                optimaPeaks = new Individual[noOfOptimaPeaks];
                for(int i = 0; i < noOfOptimaPeaks; i++)
                        optimaPeaks[i] = new Individual(functionNo, noOfVariables);
                optimaPeaks[0].setGenes(-9.929044678947709E-6, -1.6228827515991917E-4);
                optimaPeaks[1].setGenes(0.8892754861669245, -6.391508755054388E-6);
                optimaPeaks[2].setGenes(-0.8889102155536972, 1.3276676829970671E-4);
                optimaPeaks[3].setGenes(-1.782003044238401, -0.0372302208116227);
                optimaPeaks[4].setGenes(1.7831737034579276, -1.3245137896128814E-6);
                break;
            case 4:
                noOfOptimaPeaks = 5;
                optimaPeaks = new Individual[noOfOptimaPeaks];
                for(int i = 0; i < noOfOptimaPeaks; i++)
                        optimaPeaks[i] = new Individual(functionNo, noOfVariables);
                optimaPeaks[0].setGenes(4.180459827809765E-6, -6.895791068552298E-6);
                optimaPeaks[1].setGenes(1.9999785393178555, 1.9999699653023908);
                optimaPeaks[2].setGenes(-1.9999104469343796, -1.9999836174582313);
                optimaPeaks[3].setGenes(1.9993822097292575, -1.99976040930306);
                optimaPeaks[4].setGenes(-1.9998542314036407, 1.9992416823242896);
                break;
            case 5:
                noOfOptimaPeaks = 6;
                optimaPeaks = new Individual[noOfOptimaPeaks];
                for(int i = 0; i < noOfOptimaPeaks; i++)
                        optimaPeaks[i] = new Individual(functionNo, noOfVariables);
                optimaPeaks[0].setGenes(-0.0898, 0.7126);
                optimaPeaks[1].setGenes(0.0898, -0.7126);
                optimaPeaks[2].setGenes(-1.7036, 0.7961);
                optimaPeaks[3].setGenes(1.7036, -0.7961);
                optimaPeaks[4].setGenes(-1.6071, -0.5687);
                optimaPeaks[5].setGenes(1.6071, 0.5687);
                break;
            case 6:
                noOfOptimaPeaks = 10;
                optimaPeaks = new Individual[noOfOptimaPeaks];
                for(int i = 0; i < noOfOptimaPeaks; i++)
                        optimaPeaks[i] = new Individual(functionNo, noOfVariables);
                optimaPeaks[0].setGenes(-0.6057100411567423, -1.177549417829076);
                optimaPeaks[1].setGenes(1.1999967807239518, 1.1998847486771016);
                optimaPeaks[2].setGenes(0.6175789021152801, 0.8944968298697576);
                optimaPeaks[3].setGenes(0.20884852339665191, 1.1999013199005661);
                optimaPeaks[4].setGenes(0.8788289537834043, 1.1999638011167557);
                optimaPeaks[5].setGenes(1.0061423175413025, 9.679687246499487E-4);
                optimaPeaks[6].setGenes(-0.17263899349392534, -2.5503617226516008E-5);
                optimaPeaks[7].setGenes(0.5865684131938379, -0.7767919373632028);
                optimaPeaks[8].setGenes(-0.6094127225979007, 0.8068795359611105);
                optimaPeaks[9].setGenes(0.1616056192869812, -1.1999380534505975);
                break;
            case 7:
                noOfOptimaPeaks = 5;
                optimaPeaks = new Individual[noOfOptimaPeaks];
                for(int i = 0; i < noOfOptimaPeaks; i++)
                        optimaPeaks[i] = new Individual(functionNo, noOfVariables);
                optimaPeaks[0].setGene(0.1000007498188164);
                optimaPeaks[1].setGene(0.8999935259957261);
                optimaPeaks[2].setGene(0.7000131109716327);
                optimaPeaks[3].setGene(0.30001589441930276);
                optimaPeaks[4].setGene(0.49996413352279034);
                break;                
            case 8:
                noOfOptimaPeaks = 2;
                optimaPeaks = new Individual[noOfOptimaPeaks];
                for(int i = 0; i < noOfOptimaPeaks; i++)
                    optimaPeaks[i] = new Individual(functionNo, noOfVariables);
                optimaPeaks[0].setGenes(-4.99990434337427, 7.107025355908014E-4);
                optimaPeaks[1].setGenes(6.291959572955415, 14.99989504804586);
                break;
            case 9:
                noOfOptimaPeaks = 2;
                optimaPeaks = new Individual[noOfOptimaPeaks];
                for(int i = 0; i < noOfOptimaPeaks; i++)
                        optimaPeaks[i] = new Individual(functionNo, noOfVariables);
                optimaPeaks[0].setGene(19.999986712839206);
                optimaPeaks[1].setGene(3.425036340426113E-6);
                break;
            case 10:
                noOfOptimaPeaks = 2;
                optimaPeaks = new Individual[noOfOptimaPeaks];
                for(int i = 0; i < noOfOptimaPeaks; i++)
                        optimaPeaks[i] = new Individual(functionNo, noOfVariables);
                optimaPeaks[0].setGene(19.999998460848357);
                optimaPeaks[1].setGene(10.000001488782859);
                break;
            case 11:    
                noOfOptimaPeaks = 5;
                optimaPeaks = new Individual[noOfOptimaPeaks];
                for(int i = 0; i < noOfOptimaPeaks; i++)
                    optimaPeaks[i] = new Individual(functionNo, noOfVariables);
                optimaPeaks[0].setGene(9.031590232657158E-6);
                optimaPeaks[1].setGene(29.999789176886164);
                optimaPeaks[2].setGene(5.000004866143773);
                optimaPeaks[3].setGene(22.50010083311512);
                optimaPeaks[4].setGene(12.499821570534712);
                break;
            case 12:
                noOfOptimaPeaks = 1;
                optimaPeaks = new Individual[noOfOptimaPeaks];
                for(int i = 0; i < noOfOptimaPeaks; i++)
                    optimaPeaks[i] = new Individual(functionNo, noOfVariables);
                for(int i = 0; i < noOfVariables; i++)
                    optimaPeaks[0].genes[i] = 0.0;
                break;
            case 13:
                noOfOptimaPeaks = (int)Math.round(Math.pow(2,Math.round(noOfVariables/2)));
                optimaPeaks = new Individual[noOfOptimaPeaks];
                    double x1loc = 1.66206766;
                    double x2loc = 1.504150559;
                for(int i = 0; i < noOfOptimaPeaks; i++)
                        optimaPeaks[i] = new Individual(functionNo, noOfVariables);
                int work,pow2;
                for(int i=0; i < noOfOptimaPeaks; i++ ) 
                {                                                                                                                                                                                                                                    
                    work = i;
                    pow2=noOfOptimaPeaks;
                    for(int j = 0; j < noOfVariables; j+=2) 
                    {
                        pow2 /= 2;
                        if( (work / pow2) == 1 ) 
                        {
                            optimaPeaks[i].genes[j] = x1loc;
                            optimaPeaks[i].genes[j+1] = x2loc;
                            work -= pow2;
                        } 
                        else 
                        {
                            optimaPeaks[i].genes[j] = -x1loc;
                            optimaPeaks[i].genes[j+1] = -x2loc;                                                                                                                            
                        }
                    }
                }
                break;
            case 14:
            	noOfOptimaPeaks = 3;
                optimaPeaks = new Individual[noOfOptimaPeaks];
                for(int i = 0; i < noOfOptimaPeaks; i++)
                    optimaPeaks[i] = new Individual(functionNo, noOfVariables);
                optimaPeaks[0].setGenes(-Math.PI, 12.275);
                optimaPeaks[1].setGenes(Math.PI, 2.275);
                optimaPeaks[2].setGenes(9.42478, 2.475);
            	break;
            case 15:
            	noOfOptimaPeaks = 1;
            	optimaPeaks = new Individual[noOfOptimaPeaks];
            	optimaPeaks[0] = new Individual(functionNo, noOfVariables);
            	for(int i = 0; i < noOfVariables; i++)
                    optimaPeaks[0].genes[i] = 0.0;
            	break;
            case 16:
            	noOfOptimaPeaks = 18;
            	optimaPeaks = new Individual[noOfOptimaPeaks];
            	for(int i = 0; i < noOfOptimaPeaks; i++)
                    optimaPeaks[i] = new Individual(functionNo, noOfVariables);
            	optimaPeaks[0].setGenes(-0.8003, 4.85806);
            	optimaPeaks[1].setGenes(-0.8003, -1.4251);
            	optimaPeaks[2].setGenes(-0.8003, -7.70836);
            	optimaPeaks[3].setGenes(-1.4251, 5.483);
            	optimaPeaks[4].setGenes(-1.4251, -0.8003);
            	optimaPeaks[5].setGenes(-1.4251, -7.0836);
            	optimaPeaks[6].setGenes(4.858, -0.8003);
            	optimaPeaks[7].setGenes(4.858, -7.0836);
            	optimaPeaks[8].setGenes(4.858, 5.483);
                optimaPeaks[9].setGenes(5.483, -1.4251);
                optimaPeaks[10].setGenes(5.483, 4.858);
                optimaPeaks[11].setGenes(5.483, -7.7083);
                optimaPeaks[12].setGenes(-7.0836, 4.858);
                optimaPeaks[13].setGenes(-7.0836, -1.4251);
                optimaPeaks[14].setGenes(-7.0836, -7.708);
                optimaPeaks[15].setGenes(-7.7083, -0.8003);
                optimaPeaks[16].setGenes(-7.7083, -7.0836);
                optimaPeaks[17].setGenes(-7.7083, 5.483);
            	break;
            case 17:
            	noOfOptimaPeaks = 2;
            	optimaPeaks = new Individual[noOfOptimaPeaks];
            	for(int i = 0; i < noOfOptimaPeaks; i++)
                    optimaPeaks[i] = new Individual(functionNo, noOfVariables);            	
                optimaPeaks[0].setGenes(2.202876237668789, 1.570795648321596);
                optimaPeaks[1].setGenes(2.2028465007149034, 2.711561276335793);
            	break;
            case 18:
            	noOfOptimaPeaks = 2;
            	optimaPeaks = new Individual[noOfOptimaPeaks];
            	for(int i = 0; i < noOfOptimaPeaks; i++)
                    optimaPeaks[i] = new Individual(functionNo, noOfVariables);
                optimaPeaks[0].setGenes(1.697, 0);
                optimaPeaks[1].setGenes(-1.444, 0);
            	break;
            case 19:
                noOfOptimaPeaks = 6;
                optimaPeaks = new Individual[noOfOptimaPeaks];
                for(int i = 0; i < noOfOptimaPeaks; i++)
                        optimaPeaks[i] = new Individual(functionNo, noOfVariables);
                optimaPeaks[0].setGenes(-0.0898, 0.07126);
                optimaPeaks[1].setGenes(0.0898, -0.07126);
                optimaPeaks[2].setGenes(-1.7036, 0.07961);
                optimaPeaks[3].setGenes(1.7036, -0.07961);
                optimaPeaks[4].setGenes(-1.6071, -0.05687);
                optimaPeaks[5].setGenes(1.6071, 0.05687);
                break;
            case 20:
                noOfOptimaPeaks = 1;
                optimaPeaks = new Individual[noOfOptimaPeaks];
                for(int i = 0; i < noOfOptimaPeaks; i++)
                    optimaPeaks[i] = new Individual(functionNo, noOfVariables);                
                for(int i = 0; i < noOfVariables; i++)
                    optimaPeaks[0].genes[i] = 0.0;
                break;
            case 22:
                noOfOptimaPeaks = 1;
                optimaPeaks = new Individual[noOfOptimaPeaks];
                for(int i = 0; i < noOfOptimaPeaks; i++)
                    optimaPeaks[i] = new Individual(functionNo, noOfVariables);
                double[] t = {1.9005000e+000, -1.5644000e+000, -9.7880000e-001, -2.2536000e+000, 2.4990000e+000, -3.2853000e+000, 9.7590000e-001, -3.6661000e+000, 9.8500000e-002, -3.2465000e+000, 3.8060000e+000, -2.6834000e+000, -1.3701000e+000, 4.1821000e+000, 2.4856000e+000, -4.2237000e+000, 3.3653000e+000, 2.1532000e+000, -3.0929000e+000, 4.3105000e+000, -2.9861000e+000, 3.4936000e+000, -2.7289000e+000, -4.1266000e+000, -2.5900000e+000, 1.3124000e+000, -1.7990000e+000, -1.1890000e+000, -1.0530000e-001, -3.1074000e+000, -3.9641000e+000, -4.3387000e+000, 3.0705000e+000, 3.3205000e+000, -3.8178000e+000, -1.4980000e+000, -4.3807000e+000, 2.7110000e+000, -3.7956000e+000, -2.3627000e+000, 4.0086000e+000, -1.3728000e+000, -4.4362000e+000, -2.9183000e+000, -2.2457000e+000, 3.0650000e-001, -8.9240000e-001, -3.2364000e+000, -1.2521000e+000, 2.7198000e+000, -3.9787000e+000, 3.0678000e+000, -4.2400000e+000, -3.9580000e+000, 3.9479000e+000, 2.2030000e-001, -2.6124000e+000, 2.6498000e+000, -2.3256000e+000, -1.5383000e+000, 3.4760000e+000, 2.4462000e+000, 2.4575000e+000, 3.7409000e+000, -2.4887000e+000, 3.8555000e+000, -1.1426000e+000, 1.3389000e+000, 2.2323000e+000, 2.3137000e+000, -4.3370000e+000, 3.9260000e+000, 3.5905000e+000, -1.2858000e+000, -2.0113000e+000, 2.9087000e+000, 3.9278000e+000, 1.0812000e+000, -7.4610000e-001, 3.4740000e+000, 2.3036000e+000, -3.3781000e+000, -4.4910000e-001, 9.4020000e-001, -3.0583000e+000, -4.2165000e+000, -2.3604000e+000, 8.3640000e-001, 1.4773000e+000, -2.7292000e+000, -1.5904000e+000, 7.6960000e-001, 5.1640000e-001, 2.6576000e+000, -5.4270000e-001, 1.0358000e+000, 6.9260000e-001, -4.2775000e+000, -1.5911000e+000, -3.5775000e+000};
                optimaPeaks[0].setGenes(t);
                break;
            case 34://evaluation should be 360
                noOfOptimaPeaks = 1;
                optimaPeaks = new Individual[noOfOptimaPeaks];
                for(int i = 0; i < noOfOptimaPeaks; i++)
                    optimaPeaks[i] = new Individual(functionNo, noOfVariables);
                double[] t1 = {1.2141000e+000, -1.0000000e-002, 1.8864000e+000, -4.1124000e+000, 2.0627000e+000, 1.1535000e+000, 4.0653000e+000, -1.0213000e+000, 1.1986000e+000, -4.1795000e+000, -2.2924000e+000, -2.5667000e+000, 1.8881000e+000, 1.5420000e+000, 3.3243000e+000, -4.3560000e-001, -3.7297000e+000, 3.9595000e+000, -2.5571000e+000, 1.3112000e+000, -1.7149000e+000, -1.9905000e+000, -3.6150000e-001, -2.5740000e+000, -3.3399000e+000, -1.2350000e-001, -2.3588000e+000, -1.7846000e+000, -1.4377000e+000, 1.2775000e+000, -1.9487000e+000, 1.8815000e+000, 2.9909000e+000, -4.1988000e+000, -2.3918000e+000, -1.6522000e+000, -3.3576000e+000, 2.0885000e+000, -5.4260000e-001, -4.4840000e+000, 8.9630000e-001, 3.4959000e+000, 1.8402000e+000, 4.2325000e+000, 4.1250000e+000, 7.1110000e-001, -4.3325000e+000, -1.8009000e+000, -7.5500000e-002, 3.4323000e+000, 3.0661000e+000, 3.5557000e+000, -4.4331000e+000, 1.6923000e+000, -3.4752000e+000, 2.7334000e+000, -1.6906000e+000, -8.9830000e-001, -1.4713000e+000, 2.7338000e+000, 1.8929000e+000, -2.4775000e+000, 3.5147000e+000, 6.0240000e-001, -2.6482000e+000, -5.7640000e-001, 2.9721000e+000, 4.2340000e+000, -1.9843000e+000, -2.9114000e+000, 1.2712000e+000, 1.6730000e-001, 2.8046000e+000, 2.1774000e+000, -2.9080000e+000, -3.8631000e+000, -5.8630000e-001, 1.1673000e+000, 1.0729000e+000, 5.2660000e-001, -4.6470000e-001, 3.0696000e+000, -2.5510000e-001, -2.1256000e+000, 3.9083000e+000, -4.4297000e+000, -2.1308000e+000, 2.1075000e+000, 1.5273000e+000, -3.4005000e+000, -7.5490000e-001, -1.3951000e+000, 3.0333000e+000, 3.1159000e+000, -9.8630000e-001, 4.3813000e+000, 2.1197000e+000, -3.5107000e+000, -7.4880000e-001, -3.7914000e+000};
                optimaPeaks[0].setGenes(t1);
                break;
            case 50:
            	noOfOptimaPeaks = 1;
                optimaPeaks = new Individual[noOfOptimaPeaks];
                for(int i = 0; i < noOfOptimaPeaks; i++)
                    optimaPeaks[i] = new Individual(functionNo, noOfVariables); 
            	if(noOfVariables == 20)
                {
                	double[] t50 = {3.16246065811604190, 3.12833145017448370, 3.09479214084494900, 3.06145059811428410, 3.02792919854217060, 2.99382609801282040, 2.95866872595774040, 2.92184226455266010, 0.49482513593227867, 0.48835709655412185, 0.48231641293878313, 0.47664472480676473, 0.47129549301840445, 0.46623099767787296, 0.46142006012626491, 0.45683663647966960, 0.45245879070189926, 0.44826762029086792, 0.44424701500322117, 0.44038285472829569};
                	optimaPeaks[0].setGenes(t50);
                }            	
            break;

            default:
                System.out.println("The peaks are not defined in setPeaks() method for this function " + functionNo);
        }
        for(int i = 0; i < noOfOptimaPeaks; i++)
        {
            optimaPeaks[i].evaluation = eval(optimaPeaks[i]);
            fitnessEvaluations--;
        }
    }
    
    public void printThePeaks()
    {
    	for(int i = 0; i < noOfOptimaPeaks; i++)
    	{
    		optimaPeaks[i].writeValues();
    		System.out.print(eval(optimaPeaks[i]) + " ");
    	}
    	System.out.println();
    }

    private int countFreeIndividuals()
    {
        int noOfFreeIndividuals = 0;
        for(int i = 0; i < popSize; i++)
            if(c[i].seedID == -1)
                noOfFreeIndividuals++;
        Individual[] temp = new Individual[noOfFreeIndividuals];
        for(int i = 0; i < noOfFreeIndividuals; i++)
            temp[i] = new Individual(functionNo, noOfVariables);
        int j = 0;
        for(int i = 0; i < popSize; i++)
            if(c[i].seedID == -1)
            {
                for(int k = 0; k < c[i].noOfGenes; k++)
                    temp[j].genes[k] = c[i].genes[k];
                temp[j++].evaluation = c[i].evaluation;
            }
       
        return noOfFreeIndividuals;
    }

    private void determineBestEvaluation()
    {
            double best = c[0].evaluation;
            for(int i = 0; i < popSize; i++)
                    if(best < c[i].evaluation)
                            best = c[i].evaluation;
            if(generationNo < evalNo)
                    bestEvaluations[generationNo] = best;
    }
/*
    private void printForMaple(Individual ind)
    {
            System.out.print("[");
            for(int j = 0; j < ind.noOfGenes; j++)
                    if(j != ind.noOfGenes - 1)
                            System.out.print(ind.genes[j] + ", ");
                    else
                            System.out.print(ind.genes[j] + ", " + eval(ind) + "],");
            fitnessEvaluations--;
            //System.out.println();
    }
*/

    //prints several points
    private void printForR(Individual ind[])
    {
            int coordinateID = 0;
            System.out.print("xp <- c(");
            while(coordinateID < 2)//the number of dimenssions
            {
                    if(coordinateID == 1)
                            System.out.print("yp <- c(");
                    for(int i = 0; i < ind.length - 1; i++)		
                            System.out.print(ind[i].genes[coordinateID] + ", ");
                    System.out.println(ind[ind.length - 1].genes[coordinateID] + ")");
                    coordinateID++;
            }
            System.out.print("zp <- c(");
            for(int i = 0; i < ind.length - 1; i++)		
                    System.out.print(ind[i].evaluation + ", ");
            System.out.println(ind[ind.length - 1].evaluation + ")");
    }

    //prints only one point
    private void printForR(Individual ind, int id)
    {		
        System.out.println("xp" + id + " <- c(" + ind.genes[0] + ")");
        System.out.println("yp" + id + " <- c(" + ind.genes[1] + ")");						
        System.out.println("zp" + id + " <- c(" + ind.evaluation + ")");		
    }

//	prints several points
    private void printToFileForR(Individual ind[], int speciesID, BufferedWriter buffWriter)
    {	
        int coordinateID = 0;
        try
        {
            buffWriter.write("\n#This subpopulation has " + ind.length + " individuals!\n\n");
            buffWriter.write("xpop" + speciesID + " <- c(");
            while(coordinateID < 2)//the number of dimenssions
            {
                if(coordinateID == 1)
                    buffWriter.write("ypop" + speciesID + " <- c(");								
                for(int i = 0; i < ind.length - 1; i++)		
                    buffWriter.write(ind[i].genes[coordinateID] + ", ");
                buffWriter.write(ind[ind.length - 1].genes[coordinateID] + ")\n");
                coordinateID++;
            }
            buffWriter.write("zpop" + speciesID + " <- c(");
            for(int i = 0; i < ind.length - 1; i++)		
                buffWriter.write(ind[i].evaluation + ", ");
            buffWriter.write(ind[ind.length - 1].evaluation + ")\n");
        }catch(IOException e)
        {
            System.err.println("Inside printToFileForR " + e);
        }
    }

//	prints only one point
    private void printToFileForR(Individual ind, int id, BufferedWriter buffWriter)
    {	
        try
        {
            buffWriter.write("xp" + id + " <- c(" + ind.genes[0] + ")\n");
            buffWriter.write("yp" + id + " <- c(" + ind.genes[1] + ")\n");						
            buffWriter.write("zp" + id + " <- c(" + ind.evaluation + ")\n");
        }catch(IOException e)
        {
                System.err.println("Inside printToFileForR for seed " + e);
        }
    }

//	prints several points
    private void printSpeciesForR(boolean contour)
    {
        //System.out.println("Numarul de seminte in generatia " + generationNo + " este " + seeds.length);
        String fileName = "multiNationSpecies/Species/generation" + generationNo + ".txt";
        
        FileWriter fileWriter;
        BufferedWriter buffWriter;
        try
        {
            fileWriter = new FileWriter(fileName);
            buffWriter = new BufferedWriter(fileWriter);
            for(int i = 0; i < seeds.length; i++)
                printToFileForR(seeds[i], i, buffWriter);

            for(int i = 0; i < seeds.length; i++)
            {
                int count = 0; //counts how many individuals belong to this subpop
                //System.out.println("#Generation " + generationNo + ", seed " + i);
                //printForR(seeds[i], i);
                //System.out.println("\n");
                for(int j = 0; j < popSize; j++)
                    if(c[j].seedID == i)
                                count++;
                Individual[] temp = new Individual[count];
                for(int k = 0; k < count; k++)
                        temp[k] = new Individual(functionNo, noOfVariables);
                int current = 0;
                for(int j = 0; j < popSize; j++)
                    if(c[j].seedID == i)
                    {
                        for(int k = 0; k < c[j].noOfGenes; k++)
                                temp[current].genes[k] = c[j].genes[k];
                        temp[current].evaluation = c[j].evaluation;
                        temp[current++].seedID = c[j].seedID;
                    }

                if (count > 0)
                {
                    printToFileForR(temp, i, buffWriter);
                }
                else
                {
                    System.out.println("How is it possible?????");
                    System.out.println("This is the case of generation " + generationNo + ", seed " + i);
                }
            }
            if(contour)
            	buffWriter.write("\npoints(xpop0, ypop0, col = 1, pch =16)\n");
            else
            	buffWriter.write("\npoints(trans3d(xpop0, ypop0, zpop0, pm = res), col = 1, pch =16)\n");
            for(int i = 1; i < seeds.length; i++)
            	if(contour)
                	buffWriter.write("points(xpop" + i + ", ypop" + i + ", col = " + (i + 2) + ", pch =16)\n");
                else
                	buffWriter.write("points(trans3d(xpop" + i + ", ypop" + i + ", zpop" + i + ", pm = res), col = " + (i + 2) + ", pch =16)\n");
            buffWriter.write("\n");
            for(int i = 0; i < seeds.length; i++)
            	if(contour)
                	buffWriter.write("points(xp" + i + ", yp" + i + ", col = 2, pch =16)\n");
                else
                	buffWriter.write("points(trans3d(xp" + i + ", yp" + i + ", zp" + i + ", pm = res), col = 2, pch =16)\n");
            buffWriter.close();
            //System.out.println("File " + fileName + " is created.");
        }catch(IOException e)
        {
                System.err.println("Exception at initialization: " + e);
        }
    }

    public static void main(String[] arg)
    {
        String parametersfile = "parametersfile.txt";
        int averageOptima = 0, runNo = 0, all = 0, averageGenerations = 0;
        double averageBestFit = 0, time = 0, timesFree = 0;
        try {
            // load properties
            if( arg.length > 0 ) parametersfile = arg[0];                
            Properties props =  Tools.argsToProps(arg, parametersfile);


            int runs = Integer.parseInt( props.getProperty( "runs", "1" ) );
            // leave instantly if runs below 1, do not try to make sense of it
            if( runs < 1 ) return;
            int runOffset = (int)Math.round( Double.parseDouble(props.getProperty( "runOffset", "0" ) ) );
            String  runDir = props.getProperty( "runDir", "" );

            // make variant directory?
            if( props.containsKey( "variantNo" ) ) {
            try{
                int variant = Integer.parseInt( props.getProperty( "variantNo" ) );
                runDir += File.separator + new Formatter().format( "%04d", variant ).toString();
                File varDir = new File( runDir );
                varDir.mkdirs();
            } catch( NumberFormatException ex ) {
                ex.printStackTrace();
            }
        }
        runNo = runs+runOffset;
        for( int run=1+runOffset; run <= runs+runOffset; run++ ) {
            System.out.println( "starting run " + run );
            props.setProperty( CURRENT_RUN_PROP_KEY, String.valueOf( run ) );

            String outname = runDir;
            if( outname.length() > 0 ) outname += "/";
            outname += "mns" + "_" + new Formatter().format( "%04d", run ).toString() + "run.log";

            PeakLog log = new PeakLog( outname );
            log.writeLogHeader( props );
            log.writeHeader();

            Main main1 = new Main( props, log );
            int peaksFound = main1.peaksFound();
            averageOptima += peaksFound;
            averageBestFit += main1.bestFit;              
            averageGenerations += main1.generationNo;
            timesFree += main1.timesFreeIndividuals;
            time += main1.elapsedTime;
            System.out.println("When inserting seeds, " + main1.duplicatesAvoided + " duplicates were avoided.");
            System.out.println("Within this run, " + main1.seedsInserted + " seeds have been inserted.");

            if(peaksFound == main1.noOfOptimaPeaks)
                    all++;
            //System.out.println("Desired optima are:");
           // main1.printThePeaks();
            log.close();
            //main1.verifyHillValley();
        }
    } catch( Throwable th ) { th.printStackTrace(); }

    System.out.println("\n============FINAL RESULTS=============\n");
    System.out.println("No of times free individuals appeared in " + runNo + " runs: " + ((double)timesFree / (double)runNo));
    System.out.println("Average best fitness in " + runNo + " runs is " + ((double)averageBestFit / (double)runNo));
    System.out.println("The average number of peaks found in " + runNo + " runs is " + ((double)(averageOptima) / (double)(runNo)));
    System.out.println("In " + all + " out of " + runNo + " runs, all peaks were found: " + 100 * ((double)all/(double)runNo) + "%");
    System.out.println(time / (double)runNo + " seconds in average for one run.");
    System.out.println("Average number of generations in " + runNo + " runs is " + ((double)averageGenerations / (double)runNo));
    
    //Toolkit.getDefaultToolkit().beep();

    }
}
